/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLXMLSaver.java,v 1.1.1.1 2006/01/31 09:13:25 gxie Exp $
 */

package org.eclipse.eodm.owl.resource;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;
import org.eclipse.eodm.owl.resource.parser.impl.NSNameGenerator;
import org.eclipse.eodm.owl.vocabulary.OWL;
import org.eclipse.eodm.rdfs.Namespace;
import org.eclipse.eodm.rdfs.vocabulary.RDF;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;

/**
 * OWLXMLSaver
 */
public class OWLXMLSaver {
    private OWLXMLSaver() {
    }

    public static void saveToFile(OWLOntology ontology, String fileName)
            throws IOException {
        saveToStream(ontology, new FileOutputStream(fileName));
    }

    public static void saveToFile(OWLOntology ontology, String fileName,
            String charset) throws IOException {
        saveToStream(ontology, new FileOutputStream(fileName), charset);
    }

    public static void saveToStream(OWLOntology ontology, OutputStream output)
            throws IOException {
        OutputStreamWriter osWriter = new OutputStreamWriter(output);
        saveToStream(ontology, new BufferedWriter(osWriter), osWriter
                .getEncoding());
    }

    public static void saveToStream(OWLOntology ontology, OutputStream output,
            String charset) throws IOException {
        saveToStream(ontology, new BufferedWriter(new OutputStreamWriter(
                output, charset)), charset);
    }

    private static void saveToStream(OWLOntology ontology, Writer writer,
            String charset) throws IOException {
        writeRDFXMLHead(ontology, writer, charset);
        new OWLXMLSaverImpl(writer, ontology).save();
        writeRDFXMLFoot(writer);
        writer.flush();
    }

    private static void writeRDFXMLHead(OWLOntology ontology, Writer writer,
            String charset) throws IOException {
        
        //Replace all invalid characters
        if (ontology.getLocalName() !=null){
            ontology.setLocalName(replaceKeywords(ontology.getLocalName()));
            ontology.setLocalName(ontology.getLocalName().replaceAll(" ","%20"));
        }
        EList list=ontology.getOwnedNamespace();
        EList ontolist=ontology.getOWLImports();
        for (int z=0; z<ontolist.size() ;z++){
            OWLOntology onto=(OWLOntology)ontolist.get(z);
            addList(list, onto.getOwnedNamespace());
        }
        for (int i=0; i<list.size();i++){
            ((Namespace)list.get(i)).setURI(replaceKeywords(((Namespace)list.get(i)).getURI()));
        }
        
        //Firstly, re-name the abbreviation of namespaces to aviod conflict
        for ( int i=0; i<list.size()-1;i++){        
            for (int j=i+1; j<list.size() ;j++){
                if (((Namespace)(list.get(i))).getName().equals(((Namespace)(list.get(j))).getName())){
                    ((Namespace)(list.get(j))).setName(NSNameGenerator._INSTANCE.getNextKey());
                }                    
            }
        }
        
        boolean rdfNSDefined = false;
        boolean rdfsNSDefined = false;
        boolean xsdNSDefined = false;
        boolean owlNSDefined = false;

        writer.write("<?xml version=\"1.0\" encoding=\"" + charset + "\"?>\n");
        writer.write("<!DOCTYPE rdf:RDF [\n");
        for (Iterator iterator = list.iterator(); iterator
                .hasNext();) {
            Namespace ns = (Namespace) iterator.next();
            if (!ns.getURI().equals(RDFS.BNODE_NAMESPACE)
                && !ns.getName().equals("default_base_uri") && ns.getURI().indexOf("%")==-1) {
                writer.write("\t<!ENTITY "
                             + ns.getName() + " \"" + ns.getURI() + "\">\n");
                if (ns.getURI().equals(RDF.NAMESPACE))
                    rdfNSDefined = true;
                if (ns.getURI().equals(RDFS.NAMESPACE))
                    rdfsNSDefined = true;
                if (ns.getURI().equals(RDFS.XSD_NAMESPACE))
                    xsdNSDefined = true;
                if (ns.getURI().equals(OWL.NAMESPACE_URI))
                    owlNSDefined = true;
            }
        }
        if (!rdfNSDefined)
            writer.write("\t<!ENTITY rdf \"" + RDF.NAMESPACE + "\">\n");
        if (!rdfsNSDefined)
            writer.write("\t<!ENTITY rdfs \"" + RDFS.NAMESPACE + "\">\n");
        if (!xsdNSDefined)
            writer.write("\t<!ENTITY xsd \"" + RDFS.XSD_NAMESPACE + "\">\n");
        if (!owlNSDefined)
            writer.write("\t<!ENTITY owl \"" + OWL.NAMESPACE_URI + "\">\n");
        writer.write("]>\n");

        writer.write("<rdf:RDF\n");
        for (Iterator iterator = list.iterator(); iterator
                .hasNext();) {
            Namespace ns = (Namespace) iterator.next();
            if (ns.getName().length() == 0)
                writer.write("\t xmlns = \"" + ns.getURI() + "\"\n");
            else if (!ns.getURI().equals(RDFS.BNODE_NAMESPACE)
                     && !ns.getName().equals("default_base_uri")){
                if (ns.getURI().indexOf("%")>=0)
                    writer.write("\t xmlns:"
                            + ns.getName() + "=\"" + ns.getURI() + "\"\n");
                else
                    writer.write("\t xmlns:"
                             + ns.getName() + "=\"&" + ns.getName() + ";\"\n");
            }
        }
        if (!rdfNSDefined)
            writer.write("\t xmlns:rdf=\"&rdf;\"\n");
        if (!rdfsNSDefined)
            writer.write("\t xmlns:rdfs=\"&rdfs;\"\n");
        if (!xsdNSDefined)
            writer.write("\t xmlns:xsd=\"&xsd;\"\n");
        if (!owlNSDefined)
            writer.write("\t xmlns:owl=\"&owl;\"\n");
        writer.write(">\n\n");
    }

    private static void writeRDFXMLFoot(Writer writer) throws IOException {
        writer.write("</rdf:RDF>\n");
    }
    
    private static String replaceKeywords(String str) {
        str = str.replaceAll("&", "&amp;");
        str = str.replaceAll("<", "&lt;");
        str = str.replaceAll(">", "&gt;");
        str = str.replaceAll("'", "&apos;");
        str = str.replaceAll("\"", "&quot;");
        
        return str;
    }
    
    private static void addList(EList list1, EList list2){
        boolean flag=false;
        for (int i=0;i<list2.size() ;i++){
            flag=false;
            for (int j=0; j<list1.size() ; j++){
                URI uri1;
                try {
                    uri1 = new URI(((Namespace)list2.get(i)).getURI());
                } catch (URISyntaxException e) {
                    throw new OWLParserException("Invalid Namespace: "+((Namespace)list2.get(i)).getURI());
                }
                URI uri2=null;
                try {
                    uri2 = new URI(((Namespace)list1.get(j)).getURI());
                } catch (URISyntaxException e1) {
                    throw new OWLParserException("Invalid Namespace: "+((Namespace)list1.get(j)).getURI());
                }
                if (uri1.equals(uri2))
                    {
                    	flag=true;
                    	break;
                    }                
            }
        if (!flag)
            list1.add(list2.get(i));
        }
        
    }
}