/**
* <copyright>
*
* Copyright (c) 2004-2005 IBM Corporation and others.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License - v 1.0
* which accompanies this distribution, and is available at
* http://opensource.org/licenses/eclipse-1.0.txt
*
* Contributors:
*   IBM - Initial API and implementation
*
* </copyright>
*
* $Id: DuplicationRemover.java,v 1.2.4.1 2007/04/28 09:54:14 lzhang Exp $
*/

package org.eclipse.eodm.owl.resource.parser.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.eodm.owl.AllValuesFromRestriction;
import org.eclipse.eodm.owl.CardinalityRestriction;
import org.eclipse.eodm.owl.ComplementClass;
import org.eclipse.eodm.owl.EnumeratedClass;
import org.eclipse.eodm.owl.HasValueRestriction;
import org.eclipse.eodm.owl.Individual;
import org.eclipse.eodm.owl.IntersectionClass;
import org.eclipse.eodm.owl.MaxCardinalityRestriction;
import org.eclipse.eodm.owl.MinCardinalityRestriction;
import org.eclipse.eodm.owl.OWLClass;
import org.eclipse.eodm.owl.OWLDataRange;
import org.eclipse.eodm.owl.OWLRestriction;
import org.eclipse.eodm.owl.SomeValuesFromRestriction;
import org.eclipse.eodm.owl.UnionClass;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.owl.resource.parser.OWLOntoModel;
import org.eclipse.eodm.owl.vocabulary.OWL;
import org.eclipse.eodm.rdfs.PlainLiteral;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFXMLLiteral;
import org.eclipse.eodm.rdfs.TypedLiteral;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;
import org.eclipse.eodm.rdfs.vocabulary.RDF;

/**
* DuplicationRemover
*
*/
public final class DuplicationRemover {
    private DuplicationRemover() {
    }

    //1. relieves duplicated OWLDataRange definitions
    public static void relieveDuplicatedDataRanges(OWLModelDocMapping mapping) {
        OWLOntoModel ontoModel = mapping.getModel();

        List toBeRemoved = new Vector();

        Iterator keys = ontoModel.getDataRanges().keySet().iterator();
        while (keys.hasNext())
            toBeRemoved.add(ontoModel.getDataRanges().get(keys.next()));

        for (int i = 0; i < toBeRemoved.size() - 1; i++) {
            OWLDataRange range1 = (OWLDataRange) toBeRemoved.get(i);
            for (int j = i + 1; j < toBeRemoved.size();) {
                OWLDataRange range2 = (OWLDataRange) toBeRemoved.get(j);
                if (equivalentDataRange(range1, range2)) {
                    removeDuplicatedRange(range1, range2, mapping);
                    range2.getOWLOneOf().clear();
                    ontoModel.getDataRanges().remove(range2.getURI());
                    ontoModel.getOntology().getContains().remove(range2);
                    toBeRemoved.remove(j);
                } else
                    j++;
            }
        }
    }

    //2. relieves anonymous duplicated OWLClass definitions
    public static void relieveDuplicatedClasses(OWLModelDocMapping mapping) {
        OWLOntoModel ontoModel = mapping.getModel();
        List toBeRemoved = new Vector();

        Iterator keys = ontoModel.getClasses().keySet().iterator();
        while (keys.hasNext())
            toBeRemoved.add(ontoModel.getClasses().get(keys.next()));

        for (int i = 0; i < toBeRemoved.size() - 1; i++) {
            OWLClass cls1 = (OWLClass) toBeRemoved.get(i);
            for (int j = i + 1; j < toBeRemoved.size();) {
                OWLClass cls2 = (OWLClass) toBeRemoved.get(j);
                if (equivalentClass(cls1, cls2)) {
                    removeDuplicatedClass(cls1, cls2, mapping);
                    clearClassReference(cls2);
                    ontoModel.getClasses().remove(cls2.getURI());
                    removeSubReferences(cls2, mapping);
                    ontoModel.getOntology().getContains().remove(cls2);
                    toBeRemoved.remove(j);
                } else
                    j++;
            }
        }
    }

    //2. relieves duplicated restriction definitions
    public static void relieveDuplicatedRestrictions(OWLModelDocMapping mapping) {
        OWLOntoModel ontoModel = mapping.getModel();

        List toBeRemoved = new Vector();

        Iterator keys = ontoModel.getRestrictions().keySet().iterator();
        while (keys.hasNext())
            toBeRemoved.add(ontoModel.getRestrictions().get(keys.next()));

        for (int i = 0; i < toBeRemoved.size() - 1; i++) {
            OWLRestriction cls1 = (OWLRestriction) toBeRemoved.get(i);
            for (int j = i + 1; j < toBeRemoved.size();) {
                OWLRestriction cls2 = (OWLRestriction) toBeRemoved.get(j);
                if (equivalentRestriction(cls1, cls2)) {
                    removeDuplicatedClass(cls1, cls2, mapping);
                    clearRestrictionReference(cls2);
                    ontoModel.getRestrictions().remove(cls2.getURI());
                    removeSubReferences(cls2, mapping);
                    ontoModel.getOntology().getContains().remove(cls2);
                    toBeRemoved.remove(j);
                } else
                    j++;
            }
        }
    }

    private static void clearRestrictionReference(OWLRestriction res) {
        res.setOWLOnProperty(null);
        if (res instanceof AllValuesFromRestriction)
            ((AllValuesFromRestriction) res).setOWLAllValuesFrom(null);
        else if (res instanceof SomeValuesFromRestriction)
            ((SomeValuesFromRestriction) res).setOWLSomeValuesFrom(null);
        else if (res instanceof HasValueRestriction)
            ((HasValueRestriction) res).setOWLHasValue(null);
        else if (res instanceof MaxCardinalityRestriction)
            ((MaxCardinalityRestriction) res).setOWLMaxCardinality(null);
        else if (res instanceof MinCardinalityRestriction)
            ((MinCardinalityRestriction) res).setOWLMinCardinality(null);
        else
            ((CardinalityRestriction) res).setOWLCardinality(null);
    }

    private static void clearClassReference(OWLClass cls) {
        if (cls instanceof IntersectionClass)
            ((IntersectionClass) cls).getOWLIntersectionOf().clear();
        else if (cls instanceof UnionClass)
            ((UnionClass) cls).getOWLUnionOf().clear();
        else if (cls instanceof ComplementClass)
            ((ComplementClass) cls).setOWLComplementOf(null);
        else if (cls instanceof EnumeratedClass)
            ((EnumeratedClass) cls).getOWLOneOf().clear();
    }

    private static void removeSubReferences(OWLClass cls,
            OWLModelDocMapping mapping) {
        List axioms = mapping.getReferences();
        for (int i = 0; i < axioms.size();) {
            ReferenceRecorder axiom = (ReferenceRecorder) axioms.get(i);
            if (axiom.getReferingResource().getURI().equals(cls.getURI()))
                axioms.remove(i);
            else
                i++;
        }
    }

    //owl:intersectionOf owl:unionOf owl:complementOf
    //owl:equivalentClass owl:disjointWith
    //owl:oneOf(?) rdf:type
    //owl:allValuesFrom owl:someValuesFrom
    //rdfs:range rdfs:domain
    //rdfs:subClassOf
    private static void removeDuplicatedClass(OWLClass same, OWLClass cls,
            OWLModelDocMapping mapping) {
        //Updated at 2005.11.25
        holdRelationships(same, cls, mapping);

        List axioms = mapping.getReferences();
        for (int i = 0; i < axioms.size();) {
            ReferenceRecorder axiom = (ReferenceRecorder) axioms.get(i);
            if (!axiom.getReferedClass().getURI().equals(cls.getURI())) {
                i++;
                continue;
            }
            if (axiom.getProperty().getFullURI()
                    .equals(OWL.INTERSECTION_OF_URI)) {
                ((IntersectionClass) axiom.getReferingResource())
                        .getOWLIntersectionOf().remove(axiom.getReferedClass());
                ((IntersectionClass) axiom.getReferingResource())
                        .getOWLIntersectionOf().add(same);
            }
            if (axiom.getProperty().getFullURI().equals(OWL.UNION_OF_URI)) {
                ((UnionClass) axiom.getReferingResource()).getOWLUnionOf()
                        .remove(axiom.getReferedClass());
                ((UnionClass) axiom.getReferingResource()).getOWLUnionOf().add(
                        same);
            }
            if (axiom.getProperty().getFullURI().equals(OWL.COMPLEMENT_OF_URI)) {
                ((ComplementClass) axiom.getReferingResource())
                        .setOWLComplementOf(same);
            }
            if (axiom.getProperty().getFullURI().equals(
                    OWL.EQUIVALENT_CLASS_URI)) {
                ((OWLClass) axiom.getReferingResource())
                        .getOWLEquivalentClass()
                        .remove(axiom.getReferedClass());
                ((OWLClass) axiom.getReferingResource())
                        .getOWLEquivalentClass().add(same);
            }
            if (axiom.getProperty().getFullURI().equals(OWL.DISJOINT_WITH_URI)) {
                ((OWLClass) axiom.getReferingResource()).getOWLDisjointWith()
                        .remove(axiom.getReferedClass());
                ((OWLClass) axiom.getReferingResource()).getOWLDisjointWith()
                        .add(same);
            }
            if (axiom.getProperty().getFullURI().equals(RDFS.P_SUBCLASSOF_STR)) {
                ((OWLClass) axiom.getReferingResource()).getRDFSSubClassOf()
                        .remove(axiom.getReferedClass());
                ((OWLClass) axiom.getReferingResource()).getRDFSSubClassOf()
                        .add(same);
            }
            if (axiom.getProperty().getFullURI()
                    .equals(OWL.ALL_VALUES_FROM_URI)) {
                ((AllValuesFromRestriction) axiom.getReferingResource())
                        .setOWLAllValuesFrom(same);
            }
            if (axiom.getProperty().getFullURI().equals(
                    OWL.SOME_VALUES_FROM_URI)) {
                ((SomeValuesFromRestriction) axiom.getReferingResource())
                        .setOWLSomeValuesFrom(same);
            }
            if (axiom.getProperty().getFullURI().equals(RDFS.P_DOMAIN_STR)) {
                ((RDFProperty) axiom.getReferingResource()).getRDFSDomain()
                        .remove(axiom.getReferedClass());
                ((RDFProperty) axiom.getReferingResource()).getRDFSDomain()
                        .add(same);
            }
            if (axiom.getProperty().getFullURI().equals(RDFS.P_RANGE_STR)) {
                ((RDFProperty) axiom.getReferingResource()).getRDFSRange()
                        .remove(axiom.getReferedClass());
                ((RDFProperty) axiom.getReferingResource()).getRDFSRange().add(
                        same);
            }
            if (axiom.getProperty().getFullURI().equals(RDF.P_TYPE_STR)) {
                ((Individual) axiom.getReferingResource()).getRDFType()
                        .remove(axiom.getReferedClass());
                ((Individual) axiom.getReferingResource()).getRDFType().add(
                        same);
            }
            axioms.remove(i);
        }
    }

    //added at 2005.11.25
    /**
     * Keep all the relationships defined in the class to be removed
     * subClassOf
     * equivalentClass
     * disjointWith
     */
    private static void holdRelationships(OWLClass reservedCls, OWLClass removedCls,
            OWLModelDocMapping mapping) {
        //subClassOf
        if ( removedCls.getRDFSSubClassOf().size() > 0 ) {
            List reserved = new ArrayList();
            Iterator removedIt = removedCls.getRDFSSubClassOf().iterator();
            while ( removedIt.hasNext() ) {
                OWLClass removedSubCls = (OWLClass)removedIt.next();
                boolean existing = false;
                Iterator reservedIt = reservedCls.getRDFSSubClassOf().iterator();
                while ( reservedIt.hasNext() ) {
                    OWLClass reservedSubCls = (OWLClass)reservedIt.next();
                    if ( equivalentClass(reservedSubCls, removedSubCls) ) {
                        existing = true;
                        break;
                    }
                }
                if ( !existing ) {
                    reserved.add(removedSubCls);
                    mapping.addReference(new ReferenceRecorder(reservedCls,
                            OWLURIConstant.SUBCLASSOF_URIreference, removedSubCls));
                }
            }
            reservedCls.getRDFSSubClassOf().addAll(reserved);
        }
        //equivalent
        if ( removedCls.getOWLEquivalentClass().size() > 0 ) {
            List reserved = new ArrayList();
            Iterator removedIt = removedCls.getOWLEquivalentClass().iterator();
            while ( removedIt.hasNext() ) {
                OWLClass removedEquivalentCls = (OWLClass)removedIt.next();
                boolean existing = false;
                Iterator reservedIt = reservedCls.getOWLEquivalentClass().iterator();
                while ( reservedIt.hasNext() ) {
                    OWLClass reservedEquivalentCls = (OWLClass)reservedIt.next();
                    if ( equivalentClass(reservedEquivalentCls, removedEquivalentCls) ) {
                        existing = true;
                        break;
                    }
                }
                if ( !existing ) {
                    reserved.add(removedEquivalentCls);
                    mapping.addReference(new ReferenceRecorder(reservedCls,
                            OWLURIConstant.EQUIVALENT_CLASS_URIreference, removedEquivalentCls));
                }
            }
            reservedCls.getOWLEquivalentClass().addAll(reserved);
        }
        //disjointWith
        if ( removedCls.getOWLDisjointWith().size() > 0 ) {
            List reserved = new ArrayList();
            Iterator removedIt = removedCls.getOWLDisjointWith().iterator();
            while ( removedIt.hasNext() ) {
                OWLClass removedDisjointCls = (OWLClass)removedIt.next();
                boolean existing = false;
                Iterator reservedIt = reservedCls.getOWLDisjointWith().iterator();
                while ( reservedIt.hasNext() ) {
                    OWLClass reservedDisjointCls = (OWLClass)reservedIt.next();
                    if ( equivalentClass(reservedDisjointCls, removedDisjointCls) ) {
                        existing = true;
                        break;
                    }
                }
                if ( !existing ) {
                    reserved.add(removedDisjointCls);
                    mapping.addReference(new ReferenceRecorder(reservedCls,
                            OWLURIConstant.DISJOINT_WITH_URIreference, removedDisjointCls));
                }
            }
            reservedCls.getOWLDisjointWith().addAll(reserved);
        }
    }
    
    private static void removeDuplicatedRange(OWLDataRange same,
            OWLDataRange range, OWLModelDocMapping mapping) {
        List axioms = mapping.getReferences();
        for (int i = 0; i < axioms.size();) {
            ReferenceRecorder axiom = (ReferenceRecorder) axioms.get(i);
            if (!axiom.getReferedClass().getURI().equals(range.getURI())) {
                i++;
                continue;
            }
            //rdfs:range
            if (axiom.getProperty().getFullURI().equals(RDFS.P_RANGE_STR)) {
                ((RDFProperty) axiom.getReferingResource()).getRDFSRange()
                        .remove(axiom.getReferedClass());
                ((RDFProperty) axiom.getReferingResource()).getRDFSRange().add(
                        same);
                axioms.remove(i);
            }
        }
    }

    private static boolean equivalentDataRange(OWLDataRange range1,
            OWLDataRange range2) {
        if (range1 == null || range2 == null)
            return false;
        if (!range1.getNamespace().getURI().equals(RDFS.BNODE_NAMESPACE))
            if (!range2.getURI().equals(range1.getURI()))
                return false;
        if (range1.getOWLOneOf().size() != range2.getOWLOneOf().size())
            return false;
        for (int i = 0; i < range1.getOWLOneOf().size(); i++) {
            RDFSLiteral literal1 = (RDFSLiteral) range1.getOWLOneOf().get(i);
            boolean inRange2 = false;
            for (int j = 0; j < range2.getOWLOneOf().size(); j++)
                if (equivalentLiteral(literal1, (RDFSLiteral) range2
                        .getOWLOneOf().get(j)))
                    inRange2 = true;
            if (!inRange2)
                return false;
        }
        return true;
    }

    private static boolean euqivalentEnumeration(EnumeratedClass cls1,
            EnumeratedClass cls2) {
        if (cls1 == null || cls2 == null)
            return false;
        if (cls1.getOWLOneOf().size() != cls2.getOWLOneOf().size())
            return false;
        for (int i = 0; i < cls1.getOWLOneOf().size(); i++) {
            Individual inv1 = (Individual) cls1.getOWLOneOf().get(i);
            boolean inCls2 = false;
            for (int j = 0; j < cls2.getOWLOneOf().size(); j++)
                if (inv1.getURI().equals(
                        ((Individual) cls2.getOWLOneOf().get(j)).getURI()))
                    inCls2 = true;
            if (!inCls2)
                return false;
        }
        return true;
    }

    private static boolean euqivalentIntersection(IntersectionClass cls1,
            IntersectionClass cls2) {
        if (cls1 == null || cls2 == null)
            return false;
        if (cls1.getOWLIntersectionOf().size() != cls2.getOWLIntersectionOf()
                .size())
            return false;
        for (int i = 0; i < cls1.getOWLIntersectionOf().size(); i++) {
            OWLClass c1 = (OWLClass) cls1.getOWLIntersectionOf().get(i);
            boolean inCls2 = false;
            for (int j = 0; j < cls2.getOWLIntersectionOf().size(); j++)
                if (equivalentClass(c1, (OWLClass) cls2.getOWLIntersectionOf()
                        .get(j)))
                    inCls2 = true;
            if (!inCls2)
                return false;
        }
        return true;
    }

    private static boolean euqivalentUnion(UnionClass cls1, UnionClass cls2) {
        if (cls1 == null || cls2 == null)
            return false;
        if (cls1.getOWLUnionOf().size() != cls2.getOWLUnionOf().size())
            return false;
        for (int i = 0; i < cls1.getOWLUnionOf().size(); i++) {
            OWLClass c1 = (OWLClass) cls1.getOWLUnionOf().get(i);
            boolean inCls2 = false;
            for (int j = 0; j < cls2.getOWLUnionOf().size(); j++)
                if (equivalentClass(c1, (OWLClass) cls2.getOWLUnionOf().get(j)))
                    inCls2 = true;
            if (!inCls2)
                return false;
        }
        return true;
    }

    private static boolean equivalentResource(RDFSResource res1,
            RDFSResource res2) {
        if (res1 instanceof RDFSLiteral) {
            if (!(res2 instanceof RDFSLiteral))
                return false;
            return equivalentLiteral((RDFSLiteral) res1, (RDFSLiteral) res2);
        }
        if (res2 instanceof RDFSLiteral)
            return false;
        return res1.getURI().equals(res2.getURI());
    }

    /**
     * RDFSLiteral --| (lexicalForm) |-- PlainLiteral(language) | |--
     * TypedLiteral --| (datatype) |-- RDFXMLLiteral 1) Concrete type must be
     * equivalent. 2) datatype, lexical form and language must be equivalent.
     * 
     * @param literal1
     * @param literal2
     * @return
     */
    private static boolean equivalentLiteral(RDFSLiteral literal1,
            RDFSLiteral literal2) {
        if (literal1 == null || literal2 == null)
            return false;
        if (literal1 instanceof RDFXMLLiteral) {
            if (!(literal2 instanceof RDFXMLLiteral))
                return false;
            RDFXMLLiteral literal11 = (RDFXMLLiteral) literal1;
            RDFXMLLiteral literal12 = (RDFXMLLiteral) literal2;
            if (!literal11.getDatatype().getURI().equals(
                    literal12.getDatatype().getURI()))
                return false;
            return literal11.getLexicalForm() == null ? literal12
                    .getLexicalForm() == null : literal11.getLexicalForm()
                    .equals(literal12.getLexicalForm());
        }
        if (literal1 instanceof TypedLiteral) {
            if (literal2 instanceof RDFXMLLiteral
                || !(literal2 instanceof TypedLiteral))
                return false;
            TypedLiteral literal11 = (TypedLiteral) literal1;
            TypedLiteral literal12 = (TypedLiteral) literal2;
            if (!literal11.getDatatype().getURI().equals(
                    literal12.getDatatype().getURI()))
                return false;
            return literal11.getLexicalForm() == null ? literal12
                    .getLexicalForm() == null : literal11.getLexicalForm()
                    .equals(literal12.getLexicalForm());
        }
        if (literal1 instanceof PlainLiteral) {
            if (!(literal2 instanceof PlainLiteral))
                return false;
            PlainLiteral literal11 = (PlainLiteral) literal1;
            PlainLiteral literal12 = (PlainLiteral) literal2;
//            boolean langEqual = literal11.getLanguage() == null ? literal12
//                    .getLanguage() == null : literal11.getLanguage().equals(
//                    literal12.getLanguage());
            return literal11.getLexicalForm() == null ? literal12
                    .getLexicalForm() == null : literal11.getLexicalForm()
                    .equals(literal12.getLexicalForm());
        }
        if (literal2 instanceof PlainLiteral
            || literal2 instanceof TypedLiteral)
            return false;
        return literal1.getLexicalForm() == null ? literal2.getLexicalForm() == null
                : literal1.getLexicalForm().equals(literal2.getLexicalForm());
    }

    //EnueratedClass / IntersectionClass / UnionClass / ComplementClass
    private static boolean equivalentClass(RDFSClass cls1, RDFSClass cls2) {
        if (!cls1.getNamespace().getURI().equals(RDFS.BNODE_NAMESPACE))
            if (!cls2.getURI().equals(cls1.getURI()))
                return false;
        //owl:Restrictions
        if (cls1 instanceof OWLRestriction) {
            if (!(cls2 instanceof OWLRestriction))
                return false;
            return equivalentRestriction((OWLRestriction) cls1,
                    (OWLRestriction) cls2);
        }
        //owl:oneOf
        if (cls1 instanceof EnumeratedClass) {
            if (!(cls2 instanceof EnumeratedClass))
                return false;
            return euqivalentEnumeration((EnumeratedClass) cls1,
                    (EnumeratedClass) cls2);
        }
        //owl:intersectionOf
        if (cls1 instanceof IntersectionClass) {
            if (!(cls2 instanceof IntersectionClass))
                return false;
            return euqivalentIntersection((IntersectionClass) cls1,
                    (IntersectionClass) cls2);
        }
        //owl:unionOf
        if (cls1 instanceof UnionClass) {
            if (!(cls2 instanceof UnionClass))
                return false;
            return euqivalentUnion((UnionClass) cls1, (UnionClass) cls2);
        }
        //owl:complementOf
        if (cls1 instanceof ComplementClass) {
            if (!(cls2 instanceof ComplementClass))
                return false;
            return equivalentClass(((ComplementClass) cls1)
                    .getOWLComplementOf(), ((ComplementClass) cls2)
                    .getOWLComplementOf());
        }
        return cls1.getURI().equals(cls2.getURI());
    }

    //AllValuesFromRestriction / SomeValuesFromRestriction /
    // HasValueRestriction
    //MaxCardinalityRestriction / MinCardinalityRestriction /
    // CardinalityRestriction
    private static boolean equivalentRestriction(OWLRestriction res1,
            OWLRestriction res2) {
        if (!(res1.getOWLOnProperty().getURI().equals(res2.getOWLOnProperty()
                .getURI())))
            return false;

        if (res1 instanceof AllValuesFromRestriction) {
            if (!(res2 instanceof AllValuesFromRestriction))
                return false;
            return equivalentClass(((AllValuesFromRestriction) res1)
                    .getOWLAllValuesFrom(), ((AllValuesFromRestriction) res2)
                    .getOWLAllValuesFrom());
        }
        if (res1 instanceof SomeValuesFromRestriction) {
            if (!(res2 instanceof SomeValuesFromRestriction))
                return false;
            return equivalentClass(((SomeValuesFromRestriction) res1)
                    .getOWLSomeValuesFrom(), ((SomeValuesFromRestriction) res2)
                    .getOWLSomeValuesFrom());
        }
        if (res1 instanceof HasValueRestriction) {
            if (!(res2 instanceof HasValueRestriction))
                return false;
            return equivalentResource(((HasValueRestriction) res1)
                    .getOWLHasValue(), ((HasValueRestriction) res2)
                    .getOWLHasValue());
        }
        if (res1 instanceof MaxCardinalityRestriction) {
            if (!(res2 instanceof MaxCardinalityRestriction))
                return false;
            return equivalentLiteral(((MaxCardinalityRestriction) res1)
                    .getOWLMaxCardinality(), ((MaxCardinalityRestriction) res2)
                    .getOWLMaxCardinality());
        }
        if (res1 instanceof MinCardinalityRestriction) {
            if (!(res2 instanceof MinCardinalityRestriction))
                return false;
            return equivalentLiteral(((MinCardinalityRestriction) res1)
                    .getOWLMinCardinality(), ((MinCardinalityRestriction) res2)
                    .getOWLMinCardinality());
        }
        if (res1 instanceof CardinalityRestriction) {
            if (!(res2 instanceof CardinalityRestriction))
                return false;
            return equivalentLiteral(((CardinalityRestriction) res1)
                    .getOWLCardinality(), ((CardinalityRestriction) res2)
                    .getOWLCardinality());
        }
        return false;
    }
}
