/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLDocumentImpl.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.eodm.owl.resource.parser.OWLDocument;
import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;

/**
 * OWLDocumentImpl - OWLDocument Implementation.
 *  
 */
public class OWLDocumentImpl implements OWLDocument {
    /** local file in defalut */
    private boolean localFile = true;

    /**
     * The local uri of the document by wchich other documents can refer to the
     * document.
     */
    private String localURI = null;

    /**
     * The public uri of the document by which other documents can refer to the
     * document.
     */
    private String publicURI = null;

    /**
     * The encoding of the document.
     */
    private String charset = null;

    /**
     * Default base URI
     */
    private String defaultBaseURI = null;

    private InputStream inputStream = null;

    /**
     * Constructs an OWL Document.
     */
    public OWLDocumentImpl() {
        super();
    }

    /**
     * Construct an OWL Document.
     * 
     * @param localURI
     *            the document local uri
     * @param publicURI
     *            the document publid uri
     * @param localFile
     *            whether it is a local file
     * @return the document
     */
    public OWLDocumentImpl(String localURI, String publicURI, boolean localFile) {
        this.localURI = localURI;
        this.publicURI = publicURI;

        this.localFile = localFile;

        if (localURI != null)
            try {
                this.localURI = new File(localURI).getCanonicalPath();
            } catch (IOException e) {
                throw new OWLParserException("The file "
                                             + localFile + " is not found.");
            }
    }

    /**
     * Construct OWLDocument by InputStream
     * @param stream the input stream
     */
    public OWLDocumentImpl(InputStream stream) {
        this.localURI = "OWLInputStream";
        this.publicURI = null;
        this.localFile = true;
        this.inputStream = stream;
    }

    /**
     * Get inputStream
     * @return inputStream
     */
    public InputStream getOWLInputStream() {
        return inputStream;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLDocument#isLocalFile()
     */
    public boolean isLocalFile() {
        return localFile;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLDocument#getLocalURI()
     */
    public String getLocalURI() {
        return localURI;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLDocument#getPublicURI()
     */
    public String getPublicURI() {
        return publicURI;
    }

    public void setPublicURI(String URI) {
        this.publicURI = URI;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLDocument#getCharset()
     */
    public String getCharset() {
        return charset;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLDocument#setCharset(java.lang.String)
     */
    public void setCharset(String charset) {
        this.charset = charset;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLDocument#readInput()
     */
    public InputStream readInput() throws OWLParserException {
        if (inputStream != null)
            return inputStream;

        InputStream input = null;

        //the document is a local file
        if (isLocalFile())
            try {
                input = new FileInputStream(localURI);
            } catch (FileNotFoundException e) {
                throw new OWLParserException("The file "
                                             + localURI + " is not found.", e);
            }
        //the document is a web resource
        else
            try {
                input = new URL(publicURI).openStream();
            } catch (MalformedURLException e) {
                throw new OWLParserException("The document uri "
                                             + publicURI + " is malformed.", e);
            } catch (IOException e) {
                throw new OWLParserException(
                        "IO exception when open the input stream.", e);
            }

        return input;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLDocument#getDefaultBaseURI()
     */
    public String getDefaultBaseURI() {
        return defaultBaseURI;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLDocument#setDefaultBaseURI(java.lang.String)
     */
    public void setDefaultBaseURI(String baseURI) {
        this.defaultBaseURI = baseURI;
    }
}