/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLParserImpl.java,v 1.3.4.1 2007/04/28 10:06:20 lzhang Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.resource.parser.OWLDocument;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocRepository;
import org.eclipse.eodm.owl.resource.parser.OWLOntoModel;
import org.eclipse.eodm.owl.resource.parser.OWLParser;
import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;
import org.eclipse.eodm.rdf.resource.parser.RDFXMLParser;
import org.eclipse.eodm.rdf.resource.parser.element.RDFTriple;
import org.eclipse.eodm.rdf.resource.parser.exception.ParserException;
import org.eclipse.eodm.rdf.resource.parser.xml.RDFXMLParserImpl;
import org.xml.sax.InputSource;


/**
 * OWLParserImpl - OWL Parser implementation
 */
public class OWLParserImpl implements OWLParser {
    /** OWL Ontology Model & Document repository */
    private OWLModelDocRepository repository = new OWLModelDocRepositoryImpl();

    /**
     * Hashtable to keep mapping information 
     * between ontology urls and local file names
     */
    private Hashtable URLmapping=new Hashtable();
    
    /**
     * Construct an OWL Parser instance.
     */
    public OWLParserImpl() {
        super();
    }
    
    //these two lists are used to store ontology information from ontology store
    //ontology classes, properties
    List listOnto=null;
    //ontology URIs
    List listOntoURIs=null;
    
    public void setOntologyURIList(List onto, List ontoURIs){
        listOnto=onto;
        listOntoURIs=ontoURIs;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.OWLParser#addOWLDocument(org.eclipse.owl.parser.mem.OWLDocument)
     */
    public void addOWLDocument(OWLDocument document) throws OWLParserException {
        /** Document's local URI and public URI can not be null at the same time */
        if (document == null
            || document.getLocalURI() == null
            && document.getPublicURI() == null)
            throw new OWLParserException("Document is not valid.");

        /** If an existing document has been added, nothing will be done. */
        if (documentExisting(document))
            //throw new OWLParserException("Document already exists.");
            return;

        /**
         * Step 1: Identify explicitly typed elements.
         * 
         * construct blank node triple deleted
         * 
         * 1) OWL Constructs: owl:Ontology n y owl:Class n y owl:Restriction y y
         * owl:ObjectProperty n y owl:DatatypeProperty n y owl:DataRange y y
         * owl:AllDifferent y y
         * 
         * 2) RDF Constructs: rdf:Statement n y rdfs:Datatype n y rdfs:Container
         * n y (rdf:Alt/rdf:Bag/rdf:Seq) rdf:List - record uris y n (*)
         */

        /**
         * 1. Instantiate a triple analyser and initialize(pre-processing) it.
         * <br>
         * The triple analyser will catch all triples produced by RDF Parser and
         * wrap them into an OWL Model: OWLOntology.
         */
        RDFParserTripleAnalyser tripleAnalyser = new RDFParserTripleAnalyser(
                repository, document);
        tripleAnalyser.preProcessing();

        /**
         * 2. Create an RDF Parser instance, and call its parse method to parse
         * the RDF input stream into RDF triples.
         * <p>
         * The triple analyser embedded in the result handler will analyse all
         * the triples generated during the parsing process. <br>
         * Triples with "rdf:type" predicate except those related to individuals
         * will be handled in the first step and other triples are stored to be
         * handled in the following steps.
         */
        RDFXMLParser parser = new RDFXMLParserImpl();
        /** 2.1 Set the namespace and statement handler. */
        RDFParserResultHandler resultHandler = new RDFParserResultHandler(
                tripleAnalyser);
        parser.setNamespaceHandler(resultHandler);
        parser.setStatementHandler(resultHandler);
        /** 2.2 Construct the input source. */
        InputSource input = new InputSource(document.readInput());
        if (document.getCharset() != null)
            input.setEncoding(document.getCharset());
        /** 2.3 Set the default base uri to be document's uri. */
        String defaultBaseURI = document.getDefaultBaseURI();
        if (defaultBaseURI == null) {
            if (document.isLocalFile()) {
                StringBuffer buffer = new StringBuffer("file:///");
                for (int i = 0; i < document.getLocalURI().length(); i++)
                    if (document.getLocalURI().charAt(i) == '\\')
                        buffer.append('/');
                    else if (document.getLocalURI().charAt(i) != ' ')
                        buffer.append(document.getLocalURI().charAt(i));
                defaultBaseURI = buffer.toString();
            } else
                defaultBaseURI = document.getPublicURI();
            document.setDefaultBaseURI(defaultBaseURI);
            tripleAnalyser.handleNamespace("default_base_uri", defaultBaseURI);
        }
        /** 2.4 Start the parsing process: Identify all the typed elements. */
        try {
            parser.parse(input, defaultBaseURI);
        } catch (ParserException e) {
            throw new OWLParserException("RDF parsing error", e);
        }

        /**
         * 3. Post processing. 1) Ontologies Combining; 2) Ontology Storage; 2)
         * Ontology - Document relationship maintenance.
         */
        tripleAnalyser.postProcessing();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.OWLParser#addOWLDocument(java.lang.String)
     */
    public void addOWLDocument(String localFileName) throws OWLParserException {
        if (localFileName == null || localFileName.length() == 0)
            return;
        OWLDocument doc = new OWLDocumentImpl(localFileName, null, true);
        addOWLDocument(doc);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.OWLParser#parseOWLDocument(org.eclipse.owl.parser.mem.OWLDocument)
     */
    public OWLOntology parseOWLDocument(OWLDocument document)
            throws OWLParserException {
        /** 1. Add the document if it does not exist */
        boolean existing = documentExisting(document);
        if (!existing)
            addOWLDocument(document);

        OWLOntology ontology = null;

        /** 2. parse the model if it is unparsed */
        OWLModelDocMapping mapping = repository.getMappingByDocument(document);
        OWLOntoModel ontoModel = mapping.getModel();
        if (ontoModel.isParsed())
            return ontoModel.getOntology();

        /**
         * Step 2: Identify implicit typed elements.
         * 
         * construct blank node triple deleted
         * 
         * 1) OWL Constructs: owl:DeprecatedClass n y owl:DeprecatedProperty n y
         * owl:FunctionalProperty n y owl:InverseFunctionalProperty n y
         * owl:SymetricProperty n y owl:TransitiveProperty n y
         * owl:Thing(Individual) n y
         * 
         * owl:cardinality/owl:maxCardinality/owl:minCardinality n
         * owl:allValuesFrom/owl:someValuesFrom/owl:hasValue n
         * 
         * owl:intersectionOf/owl:unionOf/owl:complementOf n owl:oneOf(Class) n
         * 
         * 2) RDF Constructs: rdf:first y rdf:rest y
         */

        /** 3. parse the ontology */
        parseOntoModel(mapping);
        if (ontoModel.getImpModels().size() > 0) {
            Iterator impKeys = ontoModel.getImpModels().keySet().iterator();
            while (impKeys.hasNext()) {
                OWLOntoModel impModel = (OWLOntoModel) ontoModel.getImpModels()
                        .get(impKeys.next());
                if (!impModel.isParsed())
                    parseOntoModel(repository.getMappingByModelURI(impModel
                            .getModelURI()));
            }
        }

        /** 4. Handle axioms */
        parseOntoModelAxioms(mapping);
        ontoModel.setParsed(true);
        if (ontoModel.getImpModels().size() > 0) {
            Iterator impKeys = ontoModel.getImpModels().keySet().iterator();
            while (impKeys.hasNext()) {
                OWLOntoModel imp = (OWLOntoModel) ontoModel.getImpModels().get(
                        impKeys.next());
                if (!imp.isParsed())
                    parseOntoModelAxioms(repository.getMappingByModelURI(imp
                            .getModelURI()));
                imp.setParsed(true);
            }
        }

        /** 5. Remove duplicated Class/Data Range definitions. */
        if (mapping.getModel().getRestrictions().size() >0){
        
        DuplicationRemover.relieveDuplicatedClasses(mapping);
        
        DuplicationRemover.relieveDuplicatedRestrictions(mapping);
        
        DuplicationRemover.relieveDuplicatedDataRanges(mapping);
        
        }
        ontology = mapping.getModel().getOntology();

        return ontology;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.OWLParser#parseOWLDocument(java.lang.String)
     */
    public OWLOntology parseOWLDocument(String localFileName)
            throws OWLParserException {
        if (localFileName == null || localFileName.length() == 0)
            return null;
        OWLDocument doc = new OWLDocumentImpl(localFileName, null, true);
        return parseOWLDocument(doc);
    }

    /**
     * Determine whether the document exists in the repository.
     * 
     * @param document
     *            the OWL Document
     * @return true if it exists
     */
    private boolean documentExisting(OWLDocument document) {
        if (repository.getDocumentByLocalURI(document.getLocalURI()) != null
            || repository.getDocumentByLocalURI(document.getPublicURI()) != null
            || repository.getDocumentByPublicURI(document.getLocalURI()) != null
            || repository.getDocumentByPublicURI(document.getPublicURI()) != null)
            return true;
        return false;
    }

    /**
     * owl:imports construct. Constructs the imports graph of the ontology
     * model.
     * 
     * @param mapping
     *            the OWLOntoModel - Document mapping
     */
    private void constructImportsGraph(OWLModelDocMapping mapping) {
        impModels.clear();
        getAllImports(mapping);
        Iterator impKeys = impModels.keySet().iterator();
        while (impKeys.hasNext()) {
            OWLOntoModel impModel = (OWLOntoModel) impModels
                    .get(impKeys.next());
            if (!impModel.getModelURI()
                    .equals(mapping.getModel().getModelURI()))
                mapping.getModel().addImpModel(impModel);
        }
        impModels.clear();
    }

    Map impModels = new HashMap();

    private Map getAllImports(OWLModelDocMapping mapping) {
        /* 1. the model dosen't import any models */
        if (mapping.getImpURIs().size() == 0)
            return impModels;

        OWLOntoModel ontoModel = mapping.getModel();
        /* 2. one model imports itself */
        if (!impModels.containsKey(ontoModel.getModelURI()))
            impModels.put(ontoModel.getModelURI(), ontoModel);

        Iterator keys = mapping.getImpURIs().keySet().iterator();
        while (keys.hasNext()) {
            String impURI = (String) mapping.getImpURIs().get(keys.next());          
            
            /* 1). impURI is a web link */
            OWLDocument document = repository.getDocumentByPublicURI(impURI);
            
            /* 2). impURI is a local file name */
            if (document==null){                
                try {
                	URL url = new URL(impURI);
                	if ( url.getProtocol().equalsIgnoreCase("file") ) {
                		String file = new File(url.getPath()).getCanonicalPath();
                		document = repository.getDocumentByLocalURI(file);
                		if ( document == null ) {
                			document = new OWLDocumentImpl(url.getPath(), null, true);
                            if (listOnto!=null)
                                addOWLDocument(document, listOnto, listOntoURIs);
                            else
                                addOWLDocument(document);
                		}
                	}
                } catch ( MalformedURLException e ) {
                	throw new OWLParserException("Invalid import uri " + impURI);
                } catch ( IOException e ) {
                	throw new OWLParserException("Local file " + impURI + " is not found.");
                }
            }
            
            // 3) check if the document exists in URLmapping
            if (document==null){
                String name=(String)URLmapping.get(impURI);  
                if (name!=null){
                    document = new OWLDocumentImpl(name, impURI, true);
                    if (listOnto!=null)
                        addOWLDocument(document, listOnto, listOntoURIs);
                    else
                        addOWLDocument(document);
                }
            }
            
            /* 4). the document does not exist in repository, get it from web */
            if (document == null) {
                document = new OWLDocumentImpl(null, impURI, false);
                if (listOnto!=null)
                    addOWLDocument(document, listOnto, listOntoURIs);
                else
                    addOWLDocument(document);
            }
            OWLOntoModel impModel = repository.getMappingByDocument(document)
                    .getModel();

            /* 5). if the imported model exists */
            if (impModels.containsKey(impModel.getModelURI()))
                continue;
            impModels.put(impModel.getModelURI(), impModel);

            /* 3. add embedded imports */
            Map embedded = getAllImports(repository
                    .getMappingByModelURI(impModel.getModelURI()));
            Iterator embeddedImpKeys = embedded.keySet().iterator();
            while (embeddedImpKeys.hasNext()) {
                OWLOntoModel embeddedImpModel = (OWLOntoModel) embedded
                        .get(embeddedImpKeys.next());
                if (!impModels.containsKey(embeddedImpModel.getModelURI()))
                    impModels.put(embeddedImpModel.getModelURI(),
                            embeddedImpModel);
            }
        }
        return impModels;
    }

    /**
     * Step2: Identify implicitly typed elements.
     * 
     * @param mapping
     *            the Model - Document mapping
     */
    private void parseOntoModel(OWLModelDocMapping mapping) {
//        System.out.println("Parsing ontology step2 " +
//         mapping.getModel().getModelURI());
        constructImportsGraph(mapping);
        List unparsedTriples = mapping.getUnparsedTriples();
        for (int i = 0; i < unparsedTriples.size();) {
            RDFTriple triple = (RDFTriple) unparsedTriples.get(i);
            if (OWLTypeIdentifier._INSTANCE.identifyOWLType2(triple
                    .getSubject(), triple.getPredicate(), triple.getObject(),
                    repository, mapping)) {
                unparsedTriples.remove(i);
                continue;
            }
            if (RDFTypeIdentifier._INSTANCE.identifyRDFType2(triple
                    .getSubject(), triple.getPredicate(), triple.getObject(),
                    mapping)) {
                unparsedTriples.remove(i);
                continue;
            }
            i++;
        }
        for (int i = 0; i < unparsedTriples.size();i++) {
            RDFTriple triple = (RDFTriple) unparsedTriples.get(i);
            if (OWLTypeIdentifier._INSTANCE.identifyOWLType3(triple
                    .getSubject(), triple.getPredicate(), triple.getObject(),
                    repository, mapping)) {
                unparsedTriples.remove(i);
                continue;
            }
        }
    }

    /**
     * Step3: Handle axioms.
     * 
     * @param mapping
     *            the Model - Document mapping
     */
    private void parseOntoModelAxioms(OWLModelDocMapping mapping) {
//        System.out.println("Parsing ontology step3 " +
//         mapping.getModel().getModelURI());
        List unparsedTriples = mapping.getUnparsedTriples();
        for (int i = 0; i < unparsedTriples.size();) {
            RDFTriple triple = (RDFTriple) unparsedTriples.get(i);
            if (OWLAxiomResolver._INSTANCE.resolveOWLAxiom(triple.getSubject(),
                    triple.getPredicate(), triple.getObject(), mapping)) {
                unparsedTriples.remove(i);
                continue;
            }
            if (RDFAxiomResolver._INSTANCE.resolveRDFAxiom(triple.getSubject(),
                    triple.getPredicate(), triple.getObject(), mapping)) {
                unparsedTriples.remove(i);
                continue;
            }
            if (OWLAxiomResolver._INSTANCE.resolveOWLFact(triple.getSubject(),
                    triple.getPredicate(), triple.getObject(), mapping)) {
                unparsedTriples.remove(i);
                continue;
            }
            i++;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.OWLParser#clear()
     */
    public void clear() {
        repository.clearResources();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.OWLParser#addOWLDocument(org.eclipse.owl.resource.parser.wrapper.OWLDocument,
     *      java.util.List, java.util.List)
     */
    public void addOWLDocument(OWLDocument document, List listOnto,
            List listURIs) throws OWLParserException {
        /** Document's local URI and public URI can not be null at the same time */
        if (document == null
            || document.getLocalURI() == null
            && document.getPublicURI() == null)
            throw new OWLParserException("Document is not valid.");

        /** If an existing document has been added, nothing will be done. */
        if (documentExisting(document))
            //throw new OWLParserException("Document already exists.");
            return;

        /**
         * Step 1: Identify explicitly typed elements.
         * 
         * construct blank node triple deleted
         * 
         * 1) OWL Constructs: owl:Ontology n y owl:Class y/n y owl:Restriction y
         * y owl:ObjectProperty n y owl:DatatypeProperty n y owl:DataRange y y
         * owl:AllDifferent y y
         * 
         * 2) RDF Constructs: rdf:Statement n y rdfs:Datatype n y rdfs:Container
         * n y (rdf:Alt/rdf:Bag/rdf:Seq) rdf:List - record uris y n (*)
         */

        /**
         * 1. Instantiate a triple analyser and initialize(pre-processing) it.
         * <br>
         * The triple analyser will catch all triples produced by RDF Parser and
         * wrap them into an OWL Model: OWLOntology.
         */
        RDFParserTripleAnalyser tripleAnalyser = new RDFParserTripleAnalyser(
                repository, document);
        tripleAnalyser.preProcessing(listOnto, listURIs);

        /**
         * 2. Create an RDF Parser instance, and call its parse method to parse
         * the RDF input stream into RDF triples.
         * <p>
         * The triple analyser embedded in the result handler will analyse all
         * the triples generated during the parsing process. <br>
         * Triples with "rdf:type" predicate except those related to individuals
         * will be handled in the first step and other triples are stored to be
         * handled in the following steps.
         */
        RDFXMLParser parser = new RDFXMLParserImpl();
        /** 2.1 Set the namespace and statement handler. */
        RDFParserResultHandler resultHandler = new RDFParserResultHandler(
                tripleAnalyser);
        parser.setNamespaceHandler(resultHandler);
        parser.setStatementHandler(resultHandler);
        /** 2.2 Construct the input source. */
        InputSource input = new InputSource(document.readInput());
        if (document.getCharset() != null)
            input.setEncoding(document.getCharset());
        /** 2.3 Set the default base uri to be document's uri. */
        String defaultBaseURI = document.getDefaultBaseURI();
        if (defaultBaseURI == null) {
            if (document.isLocalFile()) {
                StringBuffer buffer = new StringBuffer("file:///");
                for (int i = 0; i < document.getLocalURI().length(); i++)
                    if (document.getLocalURI().charAt(i) == '\\')
                        buffer.append('/');
                    else if (document.getLocalURI().charAt(i) != ' ')
                        buffer.append(document.getLocalURI().charAt(i));
                defaultBaseURI = buffer.toString();
            } else
                defaultBaseURI = document.getPublicURI();
            document.setDefaultBaseURI(defaultBaseURI);
            tripleAnalyser.handleNamespace("default_base_uri", defaultBaseURI);
        }
        /** 2.4 Start the parsing process: Identify all the typed elements. */
        try {
            parser.parse(input, defaultBaseURI);
        } catch (ParserException e) {
            throw new OWLParserException("RDF parsing error", e);
        }

        /**
         * 3. Post processing. 1) Ontologies Combining; 2) Ontology Storage; 2)
         * Ontology - Document relationship maintenance.
         */
        tripleAnalyser.postProcessing();
    }

    public void addURL2FileMapping(String url, String filename) {
        String name=(String)URLmapping.get(url);
        if (name==null)
            URLmapping.put(url, filename);             
    }
}