/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFAxiomResolver.java,v 1.3 2006/12/04 06:02:31 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.util.Iterator;

import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;
import org.eclipse.eodm.rdf.resource.parser.element.RDFLiteralElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.PlainLiteral;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFStatement;
import org.eclipse.eodm.rdfs.vocabulary.RDF;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;

/**
 * RDFAxiomResolver
 *  
 */
public final class RDFAxiomResolver {
    private RDFAxiomResolver() {
    }

    public static final RDFAxiomResolver _INSTANCE = new RDFAxiomResolver();

    /**
     * rdf:subject rdf:predicate rdf:object rdfs:range rdfs:domain
     * rdfs:subClassOf rdfs:subPropertyOf
     * 
     * rdfs:label rdfs:comment rdfs:seeAlso rdfs:isDefinedBy rdf:value
     * rdfs:member
     * 
     * rdf:_n
     */
    public boolean resolveRDFAxiom(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (handleRDFSSubClassOf(subject, predicate, object, mapping))
            return true;
        if (handleRDFSubPropertyOf(subject, predicate, object, mapping))
            return true;
        if (handleRDFSDomain(subject, predicate, object, mapping))
            return true;
        if (handleRDFSRange(subject, predicate, object, mapping))
            return true;
        if (handleRDFReification(subject, predicate, object, mapping))
            return true;
        if (handleRDFUtilityProperty(subject, predicate, object, mapping))
            return true;
        if (handleRDFMemberProperty(subject, predicate, object, mapping))
            return true;
        return false;
    }

    //rdfs:subClassOf
    private boolean handleRDFSSubClassOf(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDFS.P_SUBCLASSOF_STR))
            return false;
        RDFSClass subClass = mapping.getModel().getClass(subject.getFullURI());
        if (subClass ==null) 
        	subClass = mapping.getModel().getRestriction(subject.getFullURI());
        if (subClass == null)
            throw new OWLParserException("Class ["
                                         + subject.getFullURI()
                                         + "] is not found.");

        RDFSClass objClass = OWLTypeIdentifier._INSTANCE.getClass(
                ((RDFResourceElement) object).getNamespace(),
                ((RDFResourceElement) object).getLocalName(), mapping);
        if (objClass == null)
            objClass = mapping.getModel().getRestriction(
                    ((RDFResourceElement) object).getFullURI());
        if (objClass == null)
            objClass = mapping.getModel().getDataRange(
                    ((RDFResourceElement) object).getFullURI());
        if (objClass == null)
            objClass = OWLTypeIdentifier._INSTANCE.getDatatype(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
        if (objClass == null)
            throw new OWLParserException("Class ["
                                         + ((RDFResourceElement) object)
                                                 .getFullURI()
                                         + "] is not found.");
        subClass.getRDFSSubClassOf().add(objClass);
        objClass.getSubClass().add(subClass);
        mapping.addReference(new ReferenceRecorder(subClass, new URIReference(
                RDFS.P_SUBCLASSOF_STR), objClass));
        return true;
    }

    //rdfs:subPropertyOf
    private boolean handleRDFSubPropertyOf(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDFS.P_SUBPROPERTYOF_STR))
            return false;
        RDFProperty subProperty = OWLTypeIdentifier._INSTANCE
                .getObjectProperty(subject.getNamespace(), subject
                        .getLocalName(), mapping);
        if (subProperty == null)
            subProperty = OWLTypeIdentifier._INSTANCE.getDatatypeProperty(
                    subject.getNamespace(), subject.getLocalName(), mapping);
        if (subProperty == null)
            throw new OWLParserException("Property ["
                                         + subject.getFullURI()
                                         + "] is not found.");
        RDFProperty objProperty = OWLTypeIdentifier._INSTANCE
                .getObjectProperty(
                        ((RDFResourceElement) object).getNamespace(),
                        ((RDFResourceElement) object).getLocalName(), mapping);
        if (objProperty == null)
            objProperty = OWLTypeIdentifier._INSTANCE.getDatatypeProperty(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
        if (objProperty == null)
            throw new OWLParserException("Property ["
                                         + ((RDFResourceElement) object)
                                                 .getFullURI()
                                         + "] is not found.");
        subProperty.getRDFSSubPropertyOf().add(objProperty);
        objProperty.getSubProperty().add(subProperty);
        return true;
    }

    //rdfs:domain
    private boolean handleRDFSDomain(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDFS.P_DOMAIN_STR))
            return false;
        RDFProperty property = OWLTypeIdentifier._INSTANCE.getObjectProperty(
                subject.getNamespace(), subject.getLocalName(), mapping);
        if (property == null)
            property = OWLTypeIdentifier._INSTANCE.getDatatypeProperty(subject
                    .getNamespace(), subject.getLocalName(), mapping);
        if (property == null)
            throw new OWLParserException("Property ["
                                         + subject.getFullURI()
                                         + "] is not found.");
        RDFSClass cls = OWLTypeIdentifier._INSTANCE.getClass(
                ((RDFResourceElement) object).getNamespace(),
                ((RDFResourceElement) object).getLocalName(), mapping);
        if (cls == null)
            throw new OWLParserException("Class ["
                                         + ((RDFResourceElement) object)
                                                 .getFullURI()
                                         + "] is not found.");
        property.getRDFSDomain().add(cls);
        cls.getOwnedProperty().add(property);
        mapping.addReference(new ReferenceRecorder(property, new URIReference(
                RDFS.P_DOMAIN_STR), cls));
        return true;
    }

    //rdfs:range
    public boolean handleRDFSRange(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDFS.P_RANGE_STR))
            return false;
        RDFProperty property = OWLTypeIdentifier._INSTANCE.getObjectProperty(
                subject.getNamespace(), subject.getLocalName(), mapping);
        if (property == null)
            property = OWLTypeIdentifier._INSTANCE.getDatatypeProperty(subject
                    .getNamespace(), subject.getLocalName(), mapping);
        if (property == null)
            throw new OWLParserException("Property ["
                                         + subject.getFullURI()
                                         + "] is not found.");
        RDFSClass cls = OWLTypeIdentifier._INSTANCE.getClass(
                ((RDFResourceElement) object).getNamespace(),
                ((RDFResourceElement) object).getLocalName(), mapping);
        if (cls == null)
            cls = mapping.getModel().getRestriction(
                    ((RDFResourceElement) object).getFullURI());
        if (cls == null)
            cls = mapping.getModel().getDataRange(
                    ((RDFResourceElement) object).getFullURI());
        if (cls == null)
            cls = OWLTypeIdentifier._INSTANCE.getDatatype(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
        if (cls == null
            && ((RDFResourceElement) object).getNamespace().equals(
                    RDFS.XSD_NAMESPACE)) {
            cls = RDFTypedElementGenerator._INSTANCE.createDatatype(mapping
                    .getModel().getNamespace(
                            ((RDFResourceElement) object).getNamespace()),
                    ((RDFResourceElement) object).getLocalName());
            mapping.getModel().addDatatype((RDFSDatatype) cls);
        }
        if (cls == null)
            throw new OWLParserException("Class ["
                                         + ((RDFResourceElement) object)
                                                 .getFullURI()
                                         + "] is not found.");
        property.getRDFSRange().add(cls);
        cls.getRangedByProperty().add(property);
        mapping.addReference(new ReferenceRecorder(property, new URIReference(
                RDFS.P_RANGE_STR), cls));
        return true;
    }

    //rdf:subject rdf:predicate rdf:object
    private boolean handleRDFReification(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDF.P_SUBJECT_STR)
            && !predicate.getFullURI().equals(RDF.P_PREDICATE_STR)
            && !predicate.getFullURI().equals(RDF.P_OBJECT_STR))
            return false;
        //1. subject to be a statement
        RDFStatement statement = mapping.getModel().getStatement(
                subject.getFullURI());
        statement.setReified(true);
        //2. object can be an RDFResource or a literal
        //precreated a null resource with just uri
        RDFSResource objRes = null;
        if (object instanceof RDFResourceElement)
            objRes = mapping.getModel().getResource(subject.getFullURI());
        else if (object instanceof RDFLiteralElement)
            objRes = RDFTypeIdentifier._INSTANCE.handleLiteral(
                    (RDFLiteralElement) object, mapping);
        //3. predicate {rdf:subject / rdf:predicate / rdf:object }
        if (predicate.getLocalName().equals(RDF.P_SUBJECT))
            statement.setRDFSubject(objRes);
        if (predicate.getLocalName().equals(RDF.P_PREDICATE))
            statement.setRDFPredicate(objRes);
        if (predicate.getLocalName().equals(RDF.P_OBJECT))
            statement.setRDFObject(objRes);
        return true;
    }

    private boolean handleRDFUtilityProperty(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDF.P_VALUE_STR)
            && !predicate.getFullURI().equals(RDFS.P_LABEL_STR)
            && !predicate.getFullURI().equals(RDFS.P_COMMENT_STR)
            && !predicate.getFullURI().equals(RDFS.MEMBER_STR)
            && !predicate.getFullURI().equals(RDFS.SEEALSO_STR)
            && !predicate.getFullURI().equals(RDFS.ISDEFINEDBY_STR))
            return false;
        RDFSResource subRes = mapping.getModel().getResource(
                subject.getFullURI());
        RDFSResource objRes = null;
        //rdf:value
        if (predicate.getFullURI().equals(RDF.P_VALUE_STR)) {
            if (object instanceof RDFLiteralElement){
                PlainLiteral literal = RDFSFactory.eINSTANCE.createPlainLiteral();
                literal.setLanguage(((RDFLiteralElement) object).getLanguage());
                literal.setLexicalForm(((RDFLiteralElement) object).getText());
                mapping.getModel().getOntology().getContains().add(literal);
                subRes.getRDFValue().add(literal);
                return true;
            }
            objRes = mapping.getModel().getResource(
                    ((RDFResourceElement) object).getFullURI());
            if (objRes==null)
            {
                objRes= RDFSFactory.eINSTANCE.createRDFSResource();
                objRes.setNamespace(mapping.getModel().getNamespace(((RDFResourceElement) object).getNamespace()));
                objRes.setLocalName(((RDFResourceElement) object).getLocalName());
                mapping.getModel().addRDFSResource(objRes);
            }
            subRes.getRDFValue().add(objRes);
            return true;
        }
        //rdfs:label <PlainLiteral>
        if (predicate.getFullURI().equals(RDFS.P_LABEL_STR)) {
            PlainLiteral literal = RDFSFactory.eINSTANCE.createPlainLiteral();
            PlainLiteral oldLiteral;
            literal.setLanguage(((RDFLiteralElement) object).getLanguage());
            literal.setLexicalForm(((RDFLiteralElement) object).getText());
            Iterator iter = subRes.getRDFSLabel().iterator();
            while (iter.hasNext()){
                oldLiteral=(PlainLiteral) iter.next();
                if (oldLiteral.getLexicalForm().equals(literal.getLexicalForm())&& 
                        ((oldLiteral.getLanguage()!=null && literal.getLanguage()!=null && oldLiteral.getLanguage().equals(literal.getLanguage() )))
                        || (oldLiteral.getLanguage()==null && literal.getLanguage()==null ))
                    return true;
            }   
            mapping.getModel().getOntology().getContains().add(literal);
            subRes.getRDFSLabel().add(literal);
            return true;
        }
        //rdfs:comment <PlainLiteral>
        if (predicate.getFullURI().equals(RDFS.P_COMMENT_STR)) {
            PlainLiteral literal = RDFSFactory.eINSTANCE.createPlainLiteral();
            PlainLiteral oldLiteral;
            literal.setLanguage(((RDFLiteralElement) object).getLanguage());
            literal.setLexicalForm(((RDFLiteralElement) object).getText());
            Iterator iter = subRes.getRDFSComment().iterator();
            while (iter.hasNext()){
                oldLiteral=(PlainLiteral) iter.next();
                if (oldLiteral.getLexicalForm().equals(literal.getLexicalForm())&& 
                        ((oldLiteral.getLanguage()!=null && literal.getLanguage()!=null && oldLiteral.getLanguage().equals(literal.getLanguage() )))
                        || (oldLiteral.getLanguage()==null && literal.getLanguage()==null ))
                    return true;
            }            
            mapping.getModel().getOntology().getContains().add(literal);
            subRes.getRDFSComment().add(literal);
            return true;
        }
        //rdfs:member
        if (predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.MEMBER)) {
            if (object instanceof RDFLiteralElement){
                PlainLiteral literal = RDFSFactory.eINSTANCE.createPlainLiteral();
                literal.setLanguage(((RDFLiteralElement) object).getLanguage());
                literal.setLexicalForm(((RDFLiteralElement) object).getText());
                mapping.getModel().getOntology().getContains().add(literal);
                subRes.getRDFSMember().add(literal);
                return true;
            }
            objRes = mapping.getModel().getResource(
                    ((RDFResourceElement) object).getFullURI());
            if (objRes==null)
            {
                objRes= RDFSFactory.eINSTANCE.createRDFSResource();
                objRes.setNamespace(mapping.getModel().getNamespace(((RDFResourceElement) object).getNamespace()));
                objRes.setLocalName(((RDFResourceElement) object).getLocalName());
                mapping.getModel().addRDFSResource(objRes);
            }
            subRes.getRDFSMember().add(objRes);
            return true;
        }
        //rdfs:seeAlso
        if (predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.SEEALSO)) {
            if (object instanceof RDFLiteralElement){
                PlainLiteral literal = RDFSFactory.eINSTANCE.createPlainLiteral();
                literal.setLanguage(((RDFLiteralElement) object).getLanguage());
                literal.setLexicalForm(((RDFLiteralElement) object).getText());
                mapping.getModel().getOntology().getContains().add(literal);
                subRes.getRDFSSeeAlso().add(literal);
                return true;
            }
            objRes = mapping.getModel().getResource(
                    ((RDFResourceElement) object).getFullURI());
            if (objRes==null)
            {
                objRes= RDFSFactory.eINSTANCE.createRDFSResource();
                objRes.setNamespace(mapping.getModel().getNamespace(((RDFResourceElement) object).getNamespace()));
                objRes.setLocalName(((RDFResourceElement) object).getLocalName());
                mapping.getModel().addRDFSResource(objRes);
            }
            subRes.getRDFSSeeAlso().add(objRes);
            return true;
        }
        //rdfs:isDefinedBy
        if (predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.ISDEFINEDBY)) {
            if (object instanceof RDFLiteralElement){
                PlainLiteral literal = RDFSFactory.eINSTANCE.createPlainLiteral();
                literal.setLanguage(((RDFLiteralElement) object).getLanguage());
                literal.setLexicalForm(((RDFLiteralElement) object).getText());
                mapping.getModel().getOntology().getContains().add(literal);
                subRes.getRDFSIsDefinedBy().add(literal);
                return true;
            }
            objRes = mapping.getModel().getResource(
                    ((RDFResourceElement) object).getFullURI());
            if (objRes==null)
            {
                objRes= RDFSFactory.eINSTANCE.createRDFSResource();
                objRes.setNamespace(mapping.getModel().getNamespace(((RDFResourceElement) object).getNamespace()));
                objRes.setLocalName(((RDFResourceElement) object).getLocalName());
                mapping.getModel().addRDFSResource(objRes);
            }
            subRes.getRDFSIsDefinedBy().add(objRes);
            return true;
        }
        return true;
    }

    //rdf:_n
    private boolean handleRDFMemberProperty(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().startsWith(RDF.NAMESPACE + "_"))
            return false;
        RDFSResource resource = mapping.getModel().getResource(
                subject.getFullURI());
        RDFSResource member = null;
        if (object instanceof RDFResourceElement)
            member = mapping.getModel().getResource(
                    ((RDFResourceElement) object).getFullURI());
        else if (object instanceof RDFLiteralElement)
            resource.getRDFSMember().add(
                    RDFTypeIdentifier._INSTANCE.handleLiteral(
                            (RDFLiteralElement) object, mapping));
        resource.getRDFSMember().add(member);
             
        return true;
    }
}