/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFParserTripleAnalyser.java,v 1.3 2006/06/22 05:57:33 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.util.Iterator;
import java.util.List;

import org.eclipse.eodm.owl.Individual;
import org.eclipse.eodm.owl.OWLAllDifferent;
import org.eclipse.eodm.owl.OWLAnnotationProperty;
import org.eclipse.eodm.owl.OWLClass;
import org.eclipse.eodm.owl.OWLDataRange;
import org.eclipse.eodm.owl.OWLDatatypeProperty;
import org.eclipse.eodm.owl.OWLFactory;
import org.eclipse.eodm.owl.OWLObjectProperty;
import org.eclipse.eodm.owl.OWLOntologyProperty;
import org.eclipse.eodm.owl.OWLRestriction;
import org.eclipse.eodm.owl.resource.parser.OWLDocument;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocRepository;
import org.eclipse.eodm.owl.resource.parser.OWLOntoModel;
import org.eclipse.eodm.owl.vocabulary.OWL;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFTriple;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.Namespace;
import org.eclipse.eodm.rdfs.RDFSContainer;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFStatement;
import org.eclipse.eodm.rdfs.vocabulary.RDF;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;

/**
 * RDFParserTripleAnalyser - RDF Parser Triple Analyser.
 *  
 */
public class RDFParserTripleAnalyser {
    /* The Ontology Model Mapping */
    private OWLModelDocRepository repository = null;

    /* The OWL Document */
    private OWLDocument document = null;

    /* The OWL Ontology Model */
    private OWLOntoModel ontoModel = null;

    /* The Model Documents Mapping */
    private OWLModelDocMapping modelDocMapping = null;

    /**
     * Constructs an RDFTripleAnalyser instance.
     * 
     * @param repository
     *            the Ontology Model Document repository
     * @param document
     *            the OWL Document to be parsed
     */
    public RDFParserTripleAnalyser(OWLModelDocRepository repository,
            OWLDocument document) {
        this.repository = repository;
        this.document = document;
    }

    /**
     * Handle all the namespaces generated during parsing.
     * 
     * @param abbr
     *            the namespace abbreviation name
     * @param uri
     *            the namespace uri
     */
    public Namespace handleNamespace(String abbr, String uri) {
        /* if the namespace is not found or it's not the BNode namespace, add it */
        if (!ontoModel.getNamespaces().containsKey(uri)) {
            Namespace namespace = RDFTypedElementGenerator._INSTANCE
                    .createNamespace(abbr, uri);
            ontoModel.addNamespace(namespace);
        }
        else{
            Namespace ns=(Namespace) ontoModel.getNamespaces().get( uri);
            if (ns.getName().startsWith("ns_") && !abbr.startsWith("ns_"))
                ns.setName(abbr);
        }
        return ontoModel.getNamespace(uri);
    }

    /**
     * Initialize the triple analyser. Create an OWL Ontology Model at first,
     * ignoring the case it has been present in the Ontology Model Document
     * repository; Combine all the resources in
     * {@link #postProcessing() post processing}.
     */
    public void preProcessing() {
        ontoModel = new OWLOntoModelImpl();
        ontoModel.setOntology(OWLFactory.eINSTANCE.createOWLOntology());
        modelDocMapping = new OWLModelDocMappingImpl(ontoModel);
        modelDocMapping.addDoc(document);

        handleNamespace("xsd", RDFS.XSD_NAMESPACE);
        handleNamespace("rdf", RDF.NAMESPACE);
        handleNamespace("rdfs", RDFS.NAMESPACE);
        handleNamespace("owl", OWL.NAMESPACE_URI);
        handleNamespace("bnode", RDFS.BNODE_NAMESPACE);

        ontoModel.addClass(OWLTypedElementGenerator._INSTANCE.createClass(
                ontoModel.getNamespace(OWL.NAMESPACE_URI), OWL.THING));
    }

    public void preProcessing(List listOnto, List listURIs) {
        ontoModel = new OWLOntoModelImpl();
        ontoModel.setOntology(OWLFactory.eINSTANCE.createOWLOntology());
        modelDocMapping = new OWLModelDocMappingImpl(ontoModel);
        modelDocMapping.addDoc(document);

        handleNamespace("xsd", RDFS.XSD_NAMESPACE);
        handleNamespace("rdf", RDF.NAMESPACE);
        handleNamespace("rdfs", RDFS.NAMESPACE);
        handleNamespace("owl", OWL.NAMESPACE_URI);
        handleNamespace("bnode", RDFS.BNODE_NAMESPACE);

        ontoModel.addClass(OWLTypedElementGenerator._INSTANCE.createClass(
                ontoModel.getNamespace(OWL.NAMESPACE_URI), OWL.THING));

        int i = 0;
        if (listURIs != null && listURIs.size() > 0)
            OWLTypeIdentifier._INSTANCE.setURIList(listURIs);
        if (listOnto != null && listOnto.size() > 0) {

            for (i = 0; i < listOnto.size(); i++) {
                Object obj = listOnto.get(i);
                if (obj instanceof OWLRestriction)
                    ontoModel.addRestriction((OWLRestriction) obj);
                else if (obj instanceof OWLClass)
                    ontoModel.addClass((OWLClass) obj);
                else if (obj instanceof OWLObjectProperty)
                    ontoModel.addObjectProperty((OWLObjectProperty) obj);
                else if (obj instanceof OWLDatatypeProperty)
                    ontoModel.addDatatypeProperty((OWLDatatypeProperty) obj);
                else if (obj instanceof Namespace)
                    ontoModel.addNamespace((Namespace) obj);
            }
        }
    }

    /**
     * Analyze statements produced by RDF parser.
     * 
     * @param subject
     *            the subject of the specified statement
     * @param predicate
     *            the predicate of the specified statement
     * @param object
     *            the object of the specified statement
     */
    public void analyzeStatement(RDFResourceElement subject,
            URIReference predicate, RDFValue object) {
        /** Regenerate RDF Blank Node ID */
        /*
         * if ( subject instanceof RDFBlankNode ) subject =
         * regenerateBNode((RDFBlankNode)subject); if ( object instanceof
         * RDFBlankNode ) object = regenerateBNode((RDFBlankNode)object);
         */
        /* 1. identify all the explicitly typed elements */
        if (OWLTypeIdentifier._INSTANCE.identifyOWLType1(subject, predicate,
                object, modelDocMapping))
            return;
        if (RDFTypeIdentifier._INSTANCE.identifyRDFType1(subject, predicate,
                object, modelDocMapping))
            return;

        /* 2. handle owl:imports construct */
        if (OWLTypeIdentifier._INSTANCE.identifyOWLImport(subject, predicate,
                object, modelDocMapping))
            return;

        /* 2. add the triple to unsedTriples list */
        modelDocMapping.addUnparsedTriple(new RDFTriple(subject, predicate,
                object));
    }

    /**
     * Add the Ontology Model and OWL Document to Ontology Document repository
     * after parsing if they are not present. Or combine all the generated
     * resources with the existing Ontology Model. <br>
     * Build the relationship between the Ontology Model and the OWL Document at
     * the same time. And add it to the Ontology Document repository.
     */
    public void postProcessing() {
        String modelURI = ontoModel.getModelURI();
        if (modelURI == null) {
            modelURI = document.getDefaultBaseURI();
            ontoModel.getOntology().setNamespace(
                    ontoModel.getNamespace(modelURI));
            ontoModel.getOntology().setLocalName("");
        }

        //System.out.println("Parsing ontology step1 " + modelURI);

        repository.addDocument(document);
        if (repository.getOntoModel(modelURI) == null) {
            repository.addOntoModel(ontoModel);
            repository.addModelDocMapping(modelDocMapping);
        } else {
            combiningModels(repository.getOntoModel(modelURI), ontoModel);
            ontoModel.getOntology().getOwnedNamespace().clear();
            ontoModel.getOntology().getOWLImports().clear();
            ontoModel.getOntology().getContains().clear();
            ontoModel.clearResources();
            modelDocMapping.clearResources();
        }
    }

    /* For Blank Node ID Regeneration */
    //private Map bNodeIDsMap = new HashMap();

    /**
     * Regenerate the blank node with a new blank node ID.
     * 
     * @param bNode
     *            old blank node
     * @return new blank node
     */
//    private RDFBlankNode regenerateBNode(RDFBlankNode bNode) {
//        if (bNodeIDsMap.containsKey(bNode.getID()))
//            bNode = new RDFBlankNode((String) bNodeIDsMap.get(bNode.getID()));
//        else {
//            String newID = BNodeIDGenerator._INSTANCE.getNextKey();
//            //old bNode ID - new bNode ID pair
//            bNodeIDsMap.put(bNode.getID(), newID);
//            bNode = new RDFBlankNode(newID);
//        }
//        return bNode;
//    }

    /**
     * Combine two Ontology Models.
     * 
     * @param oneModel
     *            the model to which resources to be transfered
     * @param anotherModel
     *            the model from which resources to be transfered
     */
    private void combiningModels(OWLOntoModel oneModel,
            OWLOntoModel anotherModel) {
        for (Iterator keys = anotherModel.getNamespaces().keySet().iterator(); keys
                .hasNext();)
            oneModel.addNamespace((Namespace) anotherModel.getNamespaces().get(
                    keys.next()));
        for (Iterator keys = anotherModel.getImpModels().keySet().iterator(); keys
                .hasNext();)
            oneModel.addImpModel((OWLOntoModel) anotherModel.getImpModels()
                    .get(keys.next()));
        for (Iterator keys = anotherModel.getClasses().keySet().iterator(); keys
                .hasNext();)
            oneModel.addClass((OWLClass) anotherModel.getClasses().get(
                    keys.next()));
        for (Iterator keys = anotherModel.getRestrictions().keySet().iterator(); keys
                .hasNext();)
            oneModel.addRestriction((OWLRestriction) anotherModel
                    .getRestrictions().get(keys.next()));
        for (Iterator keys = anotherModel.getObjectProperties().keySet()
                .iterator(); keys.hasNext();)
            oneModel.addObjectProperty((OWLObjectProperty) anotherModel
                    .getObjectProperties().get(keys.next()));
        for (Iterator keys = anotherModel.getDatatypeProperties().keySet()
                .iterator(); keys.hasNext();)
            oneModel.addDatatypeProperty((OWLDatatypeProperty) anotherModel
                    .getDatatypeProperties().get(keys.next()));
        for (Iterator keys = anotherModel.getIndividuals().keySet().iterator(); keys
                .hasNext();)
            oneModel.addIndividual((Individual) anotherModel.getIndividuals()
                    .get(keys.next()));
        for (Iterator keys = anotherModel.getDataRanges().keySet().iterator(); keys
                .hasNext();)
            oneModel.addDataRange((OWLDataRange) anotherModel.getDataRanges()
                    .get(keys.next()));
        for (Iterator keys = anotherModel.getAllDifferents().keySet()
                .iterator(); keys.hasNext();) {
            String key = (String) keys.next();
            oneModel.addAllDifferent(key, (OWLAllDifferent) anotherModel
                    .getAllDifferents().get(key));
        }
        for (Iterator keys = anotherModel.getStatements().keySet().iterator(); keys
                .hasNext();)
            oneModel.addStatement((RDFStatement) anotherModel.getStatements()
                    .get(keys.next()));
        for (Iterator keys = anotherModel.getDatatypes().keySet().iterator(); keys
                .hasNext();)
            oneModel.addDatatype((RDFSDatatype) anotherModel.getDatatypes()
                    .get(keys.next()));
        for (Iterator keys = anotherModel.getContainers().keySet().iterator(); keys
                .hasNext();)
            oneModel.addContainer((RDFSContainer) anotherModel.getContainers()
                    .get(keys.next()));
        for (Iterator keys = anotherModel.getRDFSResource().keySet().iterator(); keys
        		.hasNext();)
            oneModel.addRDFSResource((RDFSResource) anotherModel.getRDFSResource()
                    .get(keys.next()));
        for (Iterator keys = anotherModel.getAnnoProperties().keySet().iterator(); keys
				.hasNext();)
            oneModel.addAnnoProperty((OWLAnnotationProperty) anotherModel.getAnnoProperties()
                    .get(keys.next()));
        for (Iterator keys = anotherModel.getOntoProperties().keySet().iterator(); keys
				.hasNext();)
            oneModel.addOntoProperty((OWLOntologyProperty) anotherModel.getOntoProperties()
                    .get(keys.next()));

        OWLModelDocMapping mapping = repository.getMappingByModelURI(oneModel
                .getModelURI());
        mapping.addDoc(document);
        mapping.getRestrictions().putAll(modelDocMapping.getRestrictions());
        mapping.getLists().putAll(modelDocMapping.getLists());
        mapping.getUnparsedTriples().addAll(
                modelDocMapping.getUnparsedTriples());
        for (Iterator keys = modelDocMapping.getImpURIs().keySet().iterator(); keys
                .hasNext();)
            mapping.addImpURI((String) modelDocMapping.getImpURIs().get(
                    keys.next()));
    }
}