/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFTypeIdentifier.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.rdf.resource.parser.element.RDFLiteralElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.PlainLiteral;
import org.eclipse.eodm.rdfs.RDFList;
import org.eclipse.eodm.rdfs.RDFSContainer;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFStatement;
import org.eclipse.eodm.rdfs.TypedLiteral;
import org.eclipse.eodm.rdfs.vocabulary.RDF;

/**
 * RDFTypeIdentifier
 *  
 */
public final class RDFTypeIdentifier {
    private RDFTypeIdentifier() {
    }

    public static final RDFTypeIdentifier _INSTANCE = new RDFTypeIdentifier();

    public boolean identifyRDFType1(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDF.P_TYPE_STR))
            return false;

        /* Object must be a URIreference */
        URIReference objRef = (URIReference) object;

        /* Create different element according to the type */
        int type = RDFTypeConstants.getRDFElementType(objRef.getFullURI());
        switch (type) {
        case RDFTypeConstants.RDF_STATEMENT:
            handleStatement(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        case RDFTypeConstants.RDF_LIST:
            handleList(subject.getNamespace(), subject.getLocalName(), mapping);
            return true;
        case RDFTypeConstants.RDFS_DATATYPE:
            handleDatatype(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        case RDFTypeConstants.RDF_ALT:
        case RDFTypeConstants.RDF_BAG:
        case RDFTypeConstants.RDF_SEQ:
            handleContainer(subject.getNamespace(), subject.getLocalName(),
                    type, mapping);
            return true;
        }

        return false;
    }

    public boolean identifyRDFType2(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDF.P_FIRST_STR)
            && !predicate.getFullURI().equals(RDF.P_REST_STR))
            return false;
        handleRDFFirst(subject, predicate, object, mapping);
        handleRDFRest(subject, predicate, object, mapping);
        return true;
    }

    private void handleRDFFirst(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDF.P_FIRST_STR))
            return;
        RDFList subList = mapping.getList(subject.getLocalName());
        if (subList == null)
            subList = handleList(subject.getNamespace(),
                    subject.getLocalName(), mapping);
        if (object instanceof RDFResourceElement) {
            RDFResourceElement objResource = (RDFResourceElement) object;
            RDFSResource first = RDFSFactory.eINSTANCE.createRDFSResource();
            first.setNamespace(mapping.getModel().getNamespace(
                    objResource.getNamespace()));
            first.setLocalName(objResource.getLocalName());
            subList.setRDFFirst(first);
        } else if (object instanceof RDFLiteralElement) {
            RDFSLiteral literal = handleLiteral((RDFLiteralElement) object,
                    mapping);
            subList.setRDFFirst(literal);
        }
    }

    private void handleRDFRest(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDF.P_REST_STR))
            return;
        //subject must be a list(blank node)
        RDFList subList = mapping.getList(subject.getLocalName());
        if (subList == null)
            subList = handleList(subject.getNamespace(),
                    subject.getLocalName(), mapping);
        //object can be a list or uri reference(rdf:nil)
        RDFResourceElement objResource = (RDFResourceElement) object;
        RDFList objList = mapping.getList(objResource.getLocalName());
        if (objList == null)
            objList = handleList(objResource.getNamespace(), objResource
                    .getLocalName(), mapping);
        subList.setRDFRest(objList);
    }

    private void handleStatement(String namespace, String localName,
            OWLModelDocMapping mapping) {
        RDFStatement statement = RDFTypedElementGenerator._INSTANCE
                .createStatement(mapping.getModel().getNamespace(namespace),
                        localName);
        mapping.getModel().addStatement(statement);
    }

    private RDFList handleList(String namespace, String localName,
            OWLModelDocMapping mapping) {
        RDFList list = RDFTypedElementGenerator._INSTANCE.createList(mapping
                .getModel().getNamespace(namespace), localName);
        mapping.addList(list);
        return list;
    }

    private void handleDatatype(String namespace, String localName,
            OWLModelDocMapping mapping) {
        RDFSDatatype datatype = RDFTypedElementGenerator._INSTANCE
                .createDatatype(mapping.getModel().getNamespace(namespace),
                        localName);
        mapping.getModel().addDatatype(datatype);
    }

    private void handleContainer(String namespace, String localName, int type,
            OWLModelDocMapping mapping) {
        RDFSContainer container = null;
        switch (type) {
        case RDFTypeConstants.RDF_ALT:
            container = RDFTypedElementGenerator._INSTANCE.createAlt(mapping
                    .getModel().getNamespace(namespace), localName);
            break;
        case RDFTypeConstants.RDF_BAG:
            container = RDFTypedElementGenerator._INSTANCE.createBag(mapping
                    .getModel().getNamespace(namespace), localName);
            break;
        case RDFTypeConstants.RDF_SEQ:
            container = RDFTypedElementGenerator._INSTANCE.createSeq(mapping
                    .getModel().getNamespace(namespace), localName);
            break;
        }
        mapping.getModel().addContainer(container);
    }

    public RDFSLiteral handleLiteral(RDFLiteralElement rdfLiteral,
            OWLModelDocMapping mapping) {
        RDFSLiteral rdfsLiteral = null;
        URIReference typeRef = rdfLiteral.getDatatype();
        if (typeRef != null) {
            if (typeRef.getFullURI().equals(RDF.NAMESPACE + RDF.C_XMLLITERAL))
                rdfsLiteral = RDFSFactory.eINSTANCE.createRDFXMLLiteral();
            else
                rdfsLiteral = RDFSFactory.eINSTANCE.createTypedLiteral();
            RDFSDatatype type = mapping.getModel().getDatatype(
                    typeRef.getFullURI());
            if (type == null) {
                type = RDFTypedElementGenerator._INSTANCE
                        .createDatatype(mapping.getModel().getNamespace(
                                typeRef.getNamespace()), typeRef.getLocalName());
                mapping.getModel().addDatatype(type);
            }
            ((TypedLiteral) rdfsLiteral).setDatatype(type);
        } else if (rdfLiteral.getLanguage() != null) {
            rdfsLiteral = RDFSFactory.eINSTANCE.createPlainLiteral();
            ((PlainLiteral) rdfsLiteral).setLanguage(rdfLiteral.getLanguage());
        } else
            rdfsLiteral = RDFSFactory.eINSTANCE.createRDFSLiteral();
        rdfsLiteral.setLexicalForm(rdfLiteral.getText());
        mapping.getModel().getOntology().getContains().add(rdfsLiteral);
        return rdfsLiteral;
    }
}