/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: ECore2OWLTransformer.java,v 1.4.4.3 2007/07/26 11:33:24 lzhang Exp $
 */

package org.eclipse.eodm.owl.transformer.ecore;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.eodm.owl.CardinalityRestriction;
import org.eclipse.eodm.owl.MaxCardinalityRestriction;
import org.eclipse.eodm.owl.MinCardinalityRestriction;
import org.eclipse.eodm.owl.OWLClass;
import org.eclipse.eodm.owl.OWLDataRange;
import org.eclipse.eodm.owl.OWLDatatypeProperty;
import org.eclipse.eodm.owl.OWLFactory;
import org.eclipse.eodm.owl.OWLObjectProperty;
import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.resource.OWLXMLSaver;
import org.eclipse.eodm.owl.transformer.OWLTransformationException;
import org.eclipse.eodm.rdfs.Namespace;
import org.eclipse.eodm.rdfs.PlainLiteral;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;
import org.eclipse.eodm.rdfs.vocabulary.XSD;

public class ECore2OWLTransformer {

	public ECore2OWLTransformer() {

	}
	
	public ECore2OWLTransformer(String configFilePath) throws OWLTransformationException {
		loadProperties(configFilePath);
	}
	
	
	public void convertECore2OWL(String input) throws OWLTransformationException {
		convertECore2OWL(input, null);
	}
    
	
	public void convertECore2OWL(String input, String output) throws OWLTransformationException {
		initialize();
		
		// load ecore file
    	ResourceSet resourceSet = new ResourceSetImpl();
        resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
                .put(Resource.Factory.Registry.DEFAULT_EXTENSION,
                        new XMIResourceFactoryImpl());

    	resourceSet.getResource(URI.createFileURI(input), true);
    	
    	Resource resource = (Resource) resourceSet.getResources().get(0);
    	
        if ((resource.getContents().size() == 0)
                || !EPackage.class.isInstance(resource.getContents().get(0))) {
        	
    		OWLTransformationException e = new OWLTransformationException(
            	"Input Ecore has no ROOT EPackage : input Ecore model is invalid.");
    		throw e;
    	} 
        
        EPackage pkg = (EPackage)resource.getContents().get(0);
        
        List ontoLst = ePackage2OWLOntology(pkg);
    	
		if(!isSeperatePackage && output==null) {
			int index = input.lastIndexOf(".ecore");
			output = (index!=-1) ? input.substring(0, index) + ".owl" : input + ".owl";
		}
        saveOWLOntologyLst(ontoLst, output);
        
    }
    
    

    /**
     * Transform a ecore package into an owl ontology
     * @param ep
     * @return
     */
    private List ePackage2OWLOntology(EPackage ep){
    	ArrayList ontoLst = new ArrayList();
    	
    	// transform code here       
        OWLOntology ontology = OWLFactory.eINSTANCE.createOWLOntology();      
        String nsURI = ep.getNsURI();
        if(!nsURI.endsWith("#")){
        	nsURI += "#";
        }
        String nsPrefix = ep.getNsPrefix();
        Namespace ns = RDFSFactory.eINSTANCE.createNamespace(ontology,nsPrefix,nsURI);
        ontology.setLocalName(ep.getName());
        ontology.setNamespace(ns);
        
        // comments
        for(Iterator it= ep.getEAnnotations().iterator();it.hasNext();) {
            EAnnotation an = (EAnnotation) it.next();
            PlainLiteral comment = eAnnotation2RDFComments(an);
            ontology.getRDFSComment().add(comment);
        }

        // eclass --> owl class
        for (Iterator it = ep.getEClassifiers().iterator(); it.hasNext();) {
        	EClassifier eclassifier = (EClassifier) it.next();
        	
       	
            // transform classifier
			RDFSResource rclass = eclassifier2OWL(eclassifier);
			
			if(rclass==null) {
				continue;
			}
			
			ontology.getContains().add(rclass);
			
			if(eclassifier instanceof EClass) {
				OWLClass oclass = (OWLClass)rclass;
				
                // eSuperTypes -> RDFSSubClassOf
                EClass eclass = (EClass) eclassifier;
                for(Iterator itSuper=eclass.getESuperTypes().iterator();itSuper.hasNext();) {
                	EClassifier eSuper = (EClassifier)itSuper.next();

                    OWLClass parents = (OWLClass)eclassifier2OWL(eSuper);
                    if(parents!=null) {
                    	parents.getSubClass().add(rclass);
                    }
                }

                // EAttribute -> OWLDatatypeProperty
                for(Iterator itAtt=eclass.getEAttributes().iterator();itAtt.hasNext();) {
                    EAttribute eatt = (EAttribute) itAtt.next();
              
                    OWLDatatypeProperty dp = eattribute2DP(eatt, ontology);
                    dp.getRDFSDomain().add(oclass);
                    ontology.getContains().add(dp); 
                    
                    /*
                     * determine multiplicity: 5 situation situation
                     *      lowBound upperBound 
                     * 0..*    0          -1 
                     * m..*    m          -1 
                     * 0..n    0           n 
                     * m..n    m           n 
                     * m..m    m           m
                     */
                    int lowerBound = eatt.getLowerBound();
                    int upperBound = eatt.getUpperBound();
                    if(lowerBound > 0 ) {
                        if(upperBound==-1) {
                            // minCardinality  m..*  (m, -1)
                        	MinCardinalityRestriction min = createMinCardRestriction(ontology, dp,lowerBound);
                            oclass.getRDFSSubClassOf().add( min );
                        } else if(lowerBound==upperBound){
                            // Cardinality (m,m)-->m..m
                        	CardinalityRestriction card = createCardRestriction(ontology, dp,lowerBound);
                            oclass.getRDFSSubClassOf().add( card );
                        } else {
                            // minCardinality & maxCardinality (m,n)-->m..n 
                        	MinCardinalityRestriction min = createMinCardRestriction(ontology, dp,lowerBound);
                        	MaxCardinalityRestriction max = createMaxCardRestriction(ontology, dp, upperBound);
                            oclass.getRDFSSubClassOf().add( min );
                            oclass.getRDFSSubClassOf().add( max );
                        }
                    } else if(upperBound!=-1){
                        // maxCardinality  (0,n)-->0..n
                    	MaxCardinalityRestriction max = createMaxCardRestriction(ontology, dp, upperBound);
                        oclass.getRDFSSubClassOf().add( max );
                    }
                }

                // EReference -> OWLObjectProperty
                for(Iterator itRef=eclass.getEReferences().iterator();itRef.hasNext();) {
                    EReference eref = (EReference) itRef.next();
                    OWLObjectProperty op = eRef2OP(eref, ontology);
                    if(op==null) {
                    	continue;
                    }
                    
                    ontology.getContains().add(op);
                    
                    // domain
                    op.getRDFSDomain().add(oclass);
                    
                    // range
                    EClass erange = eref.getEReferenceType();
                    if(erange!=null) {
	                    OWLClass range = (OWLClass) eclassifier2OWL(erange);
	                    if(range!=null){
	                    	op.getRDFSRange().add(range);
	                    }
                    }
                    
                    EReference eOpposite = (EReference) eref.getEOpposite();
                    if(eOpposite != null){
                    	OWLObjectProperty oInvOf = eRef2OP(eOpposite,ontology);
                    	if(oInvOf!=null) {
                    		op.setOWLInverseOf(oInvOf);
                    	}
                    }
                    
                    /*
                     * determine multiplicity: 5 situation situation
                     *      lowBound upperBound 
                     * 0..*    0          -1 
                     * m..*    m          -1 
                     * 0..n    0           n 
                     * m..n    m           n 
                     * m..m    m           m
                     */
                    int lowerBound = eref.getLowerBound();
                    int upperBound = eref.getUpperBound();
                    if(lowerBound > 0 ) {
                        if(upperBound==-1) {
                            // minCardinality  m..*  (m, -1)
                        	MinCardinalityRestriction min = createMinCardRestriction(ontology, op,lowerBound);
                            oclass.getRDFSSubClassOf().add( min );
                        } else if(lowerBound==upperBound){
                            // Cardinality (m,m)-->m..m
                        	CardinalityRestriction card = createCardRestriction(ontology, op,lowerBound);
                            oclass.getRDFSSubClassOf().add( card );
                        } else {
                            // minCardinality & maxCardinality (m,n)-->m..n 
                        	MinCardinalityRestriction min = createMinCardRestriction(ontology, op,lowerBound);
                        	MaxCardinalityRestriction max = createMaxCardRestriction(ontology, op, upperBound);
                            oclass.getRDFSSubClassOf().add( min );
                            oclass.getRDFSSubClassOf().add( max );
                        }
                    } else if(upperBound!=-1){
                        // maxCardinality  (0,n)-->0..n
                    	MaxCardinalityRestriction max = createMaxCardRestriction(ontology, op, upperBound);
                        oclass.getRDFSSubClassOf().add( max );
                    }
                }
			}
        }
        
        ontoLst.add(ontology);
        
        //deal with subPackages
    	for(Iterator itSubP = ep.getESubpackages().iterator();itSubP.hasNext();){
    		EPackage subPkg = (EPackage) itSubP.next();
    		List subOntos = ePackage2OWLOntology(subPkg);
    		ontoLst.addAll(subOntos);
    	}
   	
    	return ontoLst;
    }
    
    private RDFSResource eclassifier2OWL(EClassifier eclassifier){
    	RDFSResource rsource = (RDFSResource)ecore2owlMap.get(eclassifier);
    	
    	if(eclassifier.eIsProxy()){
    		return null;
    	}
    	
    	if(rsource==null) {
        	//String uri = getURI(eclassifier);
        	
			if(eclassifier instanceof EClass ){
                // EClass -> OWLClass
                EClass eclass = (EClass)eclassifier;
                OWLClass oclass = OWLFactory.eINSTANCE.createOWLClass();
                oclass.setLocalName(eclass.getName());
                Namespace ns = getNamespace(eclass);
                oclass.setNamespace(ns);
                
                oclass.getRDFSComment().addAll( transformEAnnotations(eclass) );          
                rsource = oclass;
                
			} else if (eclassifier instanceof EEnum) {
                // EEnum -> OWLDataRangeOneOf
                rsource = eenum2OWLDataRange((EEnum)eclassifier);
            } else if (eclassifier instanceof EDataType) {
                // EDataType -> RDFSDatatype
                EDataType datatype = (EDataType) eclassifier;
                RDFSDatatype rdatatype = RDFSFactory.eINSTANCE.createRDFSDatatype();
                rdatatype.setLocalName(datatype.getName());
                Namespace ns = getNamespace(datatype);
                rdatatype.setNamespace(ns);
                rdatatype.getRDFSComment().addAll( transformEAnnotations(datatype) );
                
                rsource = rdatatype;
            }
			
			ecore2owlMap.put(eclassifier, rsource);
    	}

    	return rsource;
    }
    
    private OWLObjectProperty eRef2OP(EReference eref,OWLOntology ontology){
    	OWLObjectProperty op = (OWLObjectProperty) ecore2owlMap.get(eref);
    	
    	if(eref.eIsProxy()){
    		return null;
    	}
    	
    	if(op==null) {
    		op = OWLFactory.eINSTANCE.createOWLObjectProperty();
    		String localName;
    		if(isNumberSuffix){
    			localName = getUniqueName(ontology, eref.getName());
    		} else {
    			localName = eref.getEContainingClass().getName() + naming_delimiter + eref.getName();
    		}
    		
    		op.setLocalName(localName);
            op.setNamespace(getNamespace(eref));
            ecore2owlMap.put(eref, op);
    	}
    	
    	return op;
    }
    
    private MinCardinalityRestriction createMinCardRestriction(OWLOntology ontology,
            RDFProperty property, int bound) {
        // cardinality
        MinCardinalityRestriction minCr = OWLFactory.eINSTANCE
                .createMinCardinalityRestriction();
        minCr.setNamespace(aNS);

        RDFSLiteral rdfsLiteral = RDFSFactory.eINSTANCE.createRDFSLiteral();
        rdfsLiteral.setLexicalForm(Integer.toString(bound));
        ontology.getContains().add(rdfsLiteral);

        minCr.setOWLOnProperty(property);
        minCr.setOWLMinCardinality(rdfsLiteral);
        ontology.getContains().add(minCr);
        return minCr;
    }

    private MaxCardinalityRestriction createMaxCardRestriction(OWLOntology ontology,
    		RDFProperty property, int bound) {
        // cardinality
        MaxCardinalityRestriction maxCr = OWLFactory.eINSTANCE
                .createMaxCardinalityRestriction();
        maxCr.setNamespace(aNS);

        RDFSLiteral rdfsLiteral = RDFSFactory.eINSTANCE.createRDFSLiteral();
        rdfsLiteral.setLexicalForm(Integer.toString(bound));
        ontology.getContains().add(rdfsLiteral);

        maxCr.setOWLOnProperty(property);
        maxCr.setOWLMaxCardinality(rdfsLiteral);
        ontology.getContains().add(maxCr);
        return maxCr;
    }

    private CardinalityRestriction createCardRestriction(OWLOntology ontology,
    		RDFProperty property, int bound) {
        // cardinality
        CardinalityRestriction cr = OWLFactory.eINSTANCE.createCardinalityRestriction();
        cr.setNamespace(aNS);

        RDFSLiteral rdfsLiteral = RDFSFactory.eINSTANCE.createRDFSLiteral();
        rdfsLiteral.setLexicalForm(Integer.toString(bound));
        ontology.getContains().add(rdfsLiteral);

        cr.setOWLOnProperty(property);
        cr.setOWLCardinality(rdfsLiteral);
        ontology.getContains().add(cr);
        return cr;
    }
    
    
    private OWLDatatypeProperty eattribute2DP(EAttribute eatt,OWLOntology ontology){
    	OWLDatatypeProperty dp = (OWLDatatypeProperty)ecore2owlMap.get(eatt);
    	
    	if(dp == null) {
			dp = OWLFactory.eINSTANCE.createOWLDatatypeProperty();
			
    		String localName;
    		if(isNumberSuffix){
    			localName = getUniqueName(ontology, eatt.getName());
    		} else {
    			localName = eatt.getEContainingClass().getName() + naming_delimiter + eatt.getName();
    		}
    		
			dp.setLocalName( localName );
			Namespace ns = getNamespace(eatt);
			dp.setNamespace(ns);
		} 

		EClassifier etype = eatt.getEAttributeType();
		
		if (!EEnum.class.isInstance(etype) ) {
        	String xsdName = null;
            //check if XSD datatype
            if (etype == EcorePackage.eINSTANCE.getEBooleanObject())
            	xsdName = XSD.sxboolean;
            else if (etype == EcorePackage.eINSTANCE.getEFloatObject())
            	xsdName = XSD.sxfloat;
            else if (etype == EcorePackage.eINSTANCE.getEByteObject())
            	xsdName = XSD.sxbyte;
            else if (etype == EcorePackage.eINSTANCE.getEInt())
            	xsdName = XSD.sxint;
            else if (etype == EcorePackage.eINSTANCE.getELongObject())
            	xsdName = XSD.sxlong;
            else if (etype == EcorePackage.eINSTANCE.getEDoubleObject())
            	xsdName = XSD.sxdouble;
            else if (etype == EcorePackage.eINSTANCE.getEShortObject())
            	xsdName = XSD.sxshort;
            else if (etype == EcorePackage.eINSTANCE.getEIntegerObject())
            	xsdName = XSD.sinteger;
            else if (etype == EcorePackage.eINSTANCE.getEString())
            	xsdName = XSD.sxstring;

            if (xsdName != null) {
            	// xsd datatype
            	addNamespace(ontology, xsdNS);            	
            	RDFSDatatype type = RDFSFactory.eINSTANCE.createRDFSDatatype();
            	type.setNamespace(xsdNS);
            	type.setLocalName(xsdName);
            	dp.getRDFSRange().add(type);
            } else {
                // user-defined datatype           
                RDFSDatatype type = (RDFSDatatype) ecore2owlMap.get(etype);

                if (type == null) {
                	type = RDFSFactory.eINSTANCE.createRDFSDatatype();
                    type.setLocalName(etype.getName());
                    type.setNamespace(getNamespace(etype));

                    ontology.getContains().add(type);
                    ecore2owlMap.put(etype, type);
                }
                
                dp.getRDFSRange().add(type);
            }
            
        } else {
        	//@TODO, oneof
        	OWLDataRange dataRange = eenum2OWLDataRange((EEnum)etype);
        	ontology.getContains().add(dataRange);
        	dp.getRDFSRange().add(dataRange);
        }
		
    	return dp;
    }
    
    private OWLDataRange eenum2OWLDataRange(EEnum eenum) {
    	RDFSResource rsource = (RDFSResource)ecore2owlMap.get(eenum);
    	
    	if(rsource==null) {
	    	// EEum 2 Datarange
	    	OWLDataRange datarange = OWLFactory.eINSTANCE.createOWLDataRange();
	    	datarange.setLocalName(eenum.getName());
	        Namespace ns = getNamespace(eenum);
	        datarange.setNamespace(ns);
	        datarange.getRDFSComment().addAll( transformEAnnotations(eenum) );             
	
	        // EEnumLiteral -> OWLMember
	        for(Iterator et = eenum.getELiterals().iterator(); et.hasNext();) {
	            EEnumLiteral enumLiteral = (EEnumLiteral) et.next();
	            
	            RDFSLiteral literal = RDFSFactory.eINSTANCE.createRDFSLiteral();
	            literal.setLexicalForm(enumLiteral.getName());
	            datarange.getOWLOneOf().add(literal);
	        }
	        ecore2owlMap.put(eenum, datarange);
	        rsource = datarange;
    	}
    	return (OWLDataRange)rsource;
    }
    
    private List transformEAnnotations(ENamedElement eNameElem) {
    	List alist = new ArrayList();
    	
        for(Iterator itAn = eNameElem.getEAnnotations().iterator();itAn.hasNext();){
        	EAnnotation an = (EAnnotation)itAn.next();
        	if(an!=null){
        		alist.add(eAnnotation2RDFComments(an));
        	}       	
        }
    	
    	return alist;
    }
       
    private PlainLiteral eAnnotation2RDFComments(EAnnotation an) {
        String str = an.getSource() + ": ";
        EMap detail = (EMap) an.getDetails();
        for (Iterator iter = detail.keySet().iterator(); iter.hasNext();) {
            Object key = iter.next();
            str += "  " + key + ":" + detail.get(key);
        }
        
        PlainLiteral comment = RDFSFactory.eINSTANCE.createPlainLiteral();
        comment.setLexicalForm(replaceKeywords(str));
        return comment;
    }

    private static String replaceKeywords(String str) {
        str = str.replaceAll("&", "&amp;");
        str = str.replaceAll("<", "&lt;");
        str = str.replaceAll(">", "&gt;");
        str = str.replaceAll("'", "&apos;");
        str = str.replaceAll("\"", "&quot;");

        return str;
    }
    
    private Namespace getNamespace(ENamedElement elem){
		String nsUri = null;
		String nsPrefix = null;
		
		EObject nsOwner = elem;
        // use EPackage's Namespace 
        while (nsOwner != null) {
            if (nsOwner instanceof EPackage) {
                nsUri = ((EPackage) nsOwner).getNsURI();
                if(!nsUri.endsWith("#")){
                	nsUri += "#";
                }
                nsPrefix = ((EPackage) nsOwner).getNsPrefix();
                break;
            } else {
                nsOwner = nsOwner.eContainer();
            }
        }
        
        Namespace ns = (Namespace)nsMap.get(nsUri);
        if(ns==null){
        	ns = RDFSFactory.eINSTANCE.createNamespace();
        	ns.setURI(nsUri);
        	ns.setName(nsPrefix);
        	nsMap.put(nsUri, ns);
        }
        
        return ns;
    }
    
//	private static String getURI(ENamedElement elem) {
//		EObject nsOwner = elem;
//		
//		String ns = null;
//        // use EPackage's Namespace 
//        while (nsOwner != null) {
//            if (nsOwner instanceof EPackage) {
//                ns = ((EPackage) nsOwner).getNsURI(); 
//                break;
//            } else {
//                nsOwner = nsOwner.eContainer();
//            }
//        }
//        
//		return ns.endsWith("#") ? (ns + elem.getName() ): (ns + "#" + elem.getName());
//	}
	
    private String getUniqueName(OWLOntology ontology, String localName) {
        String name = localName;
        
        int i = 1;
        while (true) {
            if (!nameMap.contains(name)) {
            	nameMap.add(name);
                return name;
            }

            name = localName + naming_delimiter + i;
            i++;
        }
    }
    
    private void addNamespace(OWLOntology ontology, Namespace ns){
    	String uri = ns.getURI();
    	for(Iterator it=ontology.getOwnedNamespace().iterator();it.hasNext();){
    		Namespace n = (Namespace)it.next();
    		if(uri.equals(n.getURI())) {
    			return;
    		}
    	}
    	
    	ontology.getOwnedNamespace().add(ns);
    }
    
    void saveOWLOntology(OWLOntology ontology, String fileName) throws IOException {
        OWLXMLSaver.saveToFile(ontology, fileName, "UTF-8");
    }
    
    void saveOWLOntologyLst(List ontoLst, String output) throws OWLTransformationException {
    	if(isSeperatePackage) {
    		if(output==null){
    			output = "./";
    		}
    		
		    // save Ontology	      
	        try {
		        for(Iterator it=ontoLst.iterator();it.hasNext();){
		        	OWLOntology ontology = (OWLOntology)it.next();
		        	String outfilePath = output + "/" + ontology.getLocalName() + ".owl";
		        	saveOWLOntology(ontology, outfilePath);
		        }
	        } catch (IOException e){
	        	throw new OWLTransformationException("fail to save ontology: " + e.getMessage());
	        }
    	} else {
    		// merge all ontology together and save as one OWL
    		int length = ontoLst.size();
    		if(length>0){
    			OWLOntology ontology = (OWLOntology)ontoLst.get(0);
        		for(int i=1;i<length;i++){
        			OWLOntology onto = (OWLOntology)ontoLst.get(i);
        			ontology.getContains().addAll(onto.getContains());
        			ontology.getOwnedNamespace().addAll(onto.getOwnedNamespace());
        		}
        		try{
        			saveOWLOntology(ontology, output);
        		} catch (IOException e){
        			throw new OWLTransformationException("fail to save ontology: " + e.getMessage());
        		}
    		}
    		
    	}
    }
	
    // load property file and then set parameters
	public void loadProperties(String filename) throws OWLTransformationException{
		if(filename == null) {
			return;
		}
		
		File file = new File(filename); 
		
		try { 
			FileInputStream in = new FileInputStream(file); 
		    Properties p = new Properties(); 
		    p.load(in); 
		    in.close(); 
		    
		    // read parameters
		    // check whether it need to seperate sub packages
		    String vSepPkg = p.getProperty(KEY_SEPERATE_PACKAGE, DEFAULT_VALUE_SEPERATE_PACKAGE);
		    isSeperatePackage = vSepPkg.equalsIgnoreCase(STR_TRUE) ? true : false;
		    
		    // get property naming methods
		    String vNamingMethods = p.getProperty(KEY_NAMING, DEFAULT_VALUE_NAMING);
		    isNumberSuffix = vNamingMethods.equalsIgnoreCase(DEFAULT_VALUE_NAMING) ? true : false;
		    
		    // get delimiters
		    String defaultDelimiter = isNumberSuffix ? DELIMITER_NUM : DELIMITER_CLS;
		    naming_delimiter = p.getProperty(KEY_NAMING_DELIMITER, defaultDelimiter);
		    
		} catch (FileNotFoundException e) { 
			throw new OWLTransformationException("Can not find " + filename); 
		} catch (IOException e) { 
			throw new OWLTransformationException("Can not read config file " + filename); 
		} 
		
	}
	
	private void initialize() {	
		ecore2owlMap.clear();
		nsMap.clear();
		nameMap.clear();
	}
    
	HashMap nsMap = new HashMap();
	HashSet nameMap = new HashSet();
	HashMap ecore2owlMap = new HashMap();
	
	
	// options
    // create an ontology for each package
	String KEY_SEPERATE_PACKAGE = "SeparetePackage";
	String DEFAULT_VALUE_SEPERATE_PACKAGE = "false";
    boolean isSeperatePackage = false;
    
    static final String KEY_NAMING = "PropertyNaming";
    static final String DEFAULT_VALUE_NAMING = "number suffix";
    boolean isNumberSuffix = true;
    

    
    static final String KEY_NAMING_DELIMITER = "NamingDelimiter";
    static final String DELIMITER_NUM = "_";
    static final String DELIMITER_CLS = ".";
    String naming_delimiter = DELIMITER_NUM;
    
    String STR_TRUE = "true";
   
    private static Namespace xsdNS = RDFSFactory.eINSTANCE.createNamespace();
    private static Namespace aNS = RDFSFactory.eINSTANCE.createNamespace();
    static {
    	xsdNS.setName(XSD.SHORTNAME);
        xsdNS.setURI(XSD.NAMESPACE);
        
        aNS.setName(RDFS.BNODE_SHORTNAME);
        aNS.setURI(RDFS.BNODE_NAMESPACE);
    }
}
