/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFCollectionConstructor.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdf.resource.parser.wrapper;

import org.eclipse.eodm.rdf.resource.parser.element.RDFLiteralElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFSURIConstant;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.RDFList;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.vocabulary.RDF;

/**
 * RDFCollectionConstructor
 */
public final class RDFCollectionConstructor {
    private RDFCollectionConstructor() {
    }

    /** The single instance of class RDFCollectionConstructor */
    public static final RDFCollectionConstructor _INSTANCE = new RDFCollectionConstructor();

    /**
     * Handle triple with the predicate of rdf:first/rdf:rest.
     * 
     * @param subject
     *            the subject of the rdf triple
     * @param predicate
     *            the predicate of the rdf triple
     * @param object
     *            the object of the rdf triple
     * @param analyser
     *            the triple analyser
     * @return true if the triple is handled
     */
    public boolean handleRDFListTriple(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        boolean handled = handleRDFFirstTriple(subject, predicate, object,
                analyser);

        if (!handled)
            handled = handleRDFRestTriple(subject, predicate, object, analyser);

        return handled;
    }

    /**
     * Haldle triple <subject rdf:first object>. The subject of the type of
     * triple must be an rdf list.
     * 
     * @param subject
     *            the subject of the triple
     * @param predicate
     *            the predicate of the triple
     * @param object
     *            the object of the triple
     * @param analyser
     *            the triple analyser
     * @return true if the triple is handled
     */
    private boolean handleRDFFirstTriple(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().equals(RDF.P_FIRST_STR))
            return false;

        //subject must be a list(often a blank node)
        RDFList subList = RDFTypeIdentifier._INSTANCE.constructRDFList(subject
                .getNamespace(), subject.getLocalName(), analyser);

        //precreated a null resource with just uri
        if (object instanceof RDFResourceElement) {
            RDFResourceElement objResource = (RDFResourceElement) object;
            RDFSResource first = RDFSFactory.eINSTANCE.createRDFSResource();
            first.setNamespace(analyser.getNamespaceByURI(objResource
                    .getNamespace()));
            first.setLocalName(objResource.getLocalName());
            subList.setRDFFirst(first);
        } else if (object instanceof RDFLiteralElement) {
            RDFSLiteral literal = analyser
                    .createLiteral((RDFLiteralElement) object);
            subList.setRDFFirst(literal);

        }

        return true;
    }

    /**
     * Handle triple <subject rdf:rest object>. The subject of the type of
     * triple must be an rdf list.
     * 
     * @param subject
     *            the subject of the triple
     * @param predicate
     *            the predicate of the triple
     * @param object
     *            the object of the triple
     * @param analyser
     *            the triple analyser
     * @return true if the triple is handled
     */
    private boolean handleRDFRestTriple(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().equals(RDF.P_REST_STR))
            return false;

        //subject must be a list(blank node)
        RDFList subList = RDFTypeIdentifier._INSTANCE.constructRDFList(subject
                .getNamespace(), subject.getLocalName(), analyser);

        //object can be a list or uri reference(rdf:nil)
        RDFResourceElement objResource = (RDFResourceElement) object;
        RDFList objList = RDFTypeIdentifier._INSTANCE.constructRDFList(
                objResource.getNamespace(), objResource.getLocalName(),
                analyser);
        subList.setRDFRest(objList);

        return true;
    }

    /**
     * Replace rdf lists' first resources with existing resources.
     * 
     * @param list
     *            the rdf list to be handled
     * @param analyser
     *            the triple analyser
     */
    public void handleRDFListFirstElelement(RDFList list,
            RDFTripleAnalyser analyser) {
        RDFSResource first = list.getRDFFirst();

        if (first != null
            && !(first instanceof RDFSLiteral)
            && !list.getLocalName().equals(RDFSURIConstant.NIL)) {
            RDFSResource defined = analyser.getRepository().getGlobalResource(
                    first.getURI(), RDFTypeIdentifier.RESOURCE_SEARCH_ORDER);
            if (defined == null) {
                analyser.getDescription().getContains().add(first);
                analyser.getRepository().addResource(first);
            } else
                list.setRDFFirst(defined);
        }
    }
}