/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFContainerConstructor.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdf.resource.parser.wrapper;

import org.eclipse.eodm.rdf.resource.parser.element.RDFLiteralElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.vocabulary.RDF;

/**
 * RDFContainerConstructor RDFS container constructor.
 */
public final class RDFContainerConstructor {
    private RDFContainerConstructor() {
    }

    /** The single instance of class RDFContainerConstructor */
    public static final RDFContainerConstructor _INSTANCE = new RDFContainerConstructor();

    /** The order in which to find the resource generated before */
    public static final int[] CONTAINER_RESOURCE_SEARCH_ORDER = new int[] {
            RDFResourceMap.CONTAINER, RDFResourceMap.RESOURCE,
            RDFResourceMap.CLASS, RDFResourceMap.PROPERTY, RDFResourceMap.LIST,
            RDFResourceMap.REFIED_STATEMENT, RDFResourceMap.DATATYPE };

    /**
     * Handle triple with "rdf:_n" predicate The predicate of rdf:_n indicates
     * that the RDFSubject must be an rdfs Container.
     * 
     * @param subject
     *            the subject of the rdf triple
     * @param predicate
     *            the predicate of the rdf triple
     * @param object
     *            the object of the rdf triple
     * @param analyser
     *            the triple analyser
     * @return true if the triple is handled
     */
    public boolean handleRDFSContainerTriple(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().startsWith(RDF.NAMESPACE + "_"))
            return false;

        //subject can be a container or a resource
        RDFSResource resource = analyser.getRepository().getGlobalResource(
                subject.getNamespace() + subject.getLocalName(),
                CONTAINER_RESOURCE_SEARCH_ORDER);
        if (resource == null) {
            resource = RDFSFactory.eINSTANCE.createRDFSResource();
            resource.setNamespace(analyser.getNamespaceByURI(subject
                    .getNamespace()));
            resource.setLocalName(subject.getLocalName());
            analyser.getDescription().getContains().add(resource);
            analyser.getRepository().addResource(resource);
        }

        //precreated a null resource just with uri
        if (object instanceof RDFResourceElement) {
            RDFResourceElement objResource = (RDFResourceElement) object;
            RDFSResource member = RDFSFactory.eINSTANCE.createRDFSResource();
            member.setNamespace(analyser.getNamespaceByURI(objResource
                    .getNamespace()));
            member.setLocalName(objResource.getLocalName());
            resource.getRDFSMember().add(member);
        } else if (object instanceof RDFLiteralElement) {
            RDFSLiteral literal = analyser
                    .createLiteral((RDFLiteralElement) object);
            resource.getRDFSMember().add(literal);
        }

        return true;
    }

    /**
     * Handle all the rdfs members resource, replace it with the found resource
     * in the resource mapping repository.
     * 
     * @param resource
     *            the resource with uri
     * @param analyser
     *            the triple analyser
     */
    public void handleRDFSMembers(RDFSResource resource,
            RDFTripleAnalyser analyser) {
        int memberCount = resource.getRDFSMember().size();
        for (int i = 0; i < memberCount; i++) {
            RDFSResource member = (RDFSResource) resource.getRDFSMember()
                    .get(i);
            if (!(member instanceof RDFSLiteral)) {
                RDFSResource defined = analyser.getRepository()
                        .getGlobalResource(member.getURI(),
                                RDFTypeIdentifier.RESOURCE_SEARCH_ORDER);
                if (defined == null) {
                    analyser.getRepository().addResource(member);
                    analyser.getDescription().getContains().add(member);
                } else
                    resource.getRDFSMember().set(i, defined);
            }
        }
    }
}