/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFResourceMap.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdf.resource.parser.wrapper;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.eodm.rdfs.RDFList;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSContainer;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFStatement;

/**
 * RDFResourceMap keeps all Hashmaps used in parsing process
 */
public class RDFResourceMap {
    //0. rdfs resources
    public static final int RESOURCE = 0;

    private Map resources = new HashMap();

    public void addResource(RDFSResource resource) {
        String uri = resource.getNamespace().getURI() + resource.getLocalName();
        addResource(uri, resource);
    }

    public void addResource(String uri, RDFSResource resource) {
        resources.put(uri, resource);
    }

    public RDFSResource getResource(String uri) {
        return (RDFSResource) resources.get(uri);
    }

    public RDFSResource removeResource(String uri) {
        return (RDFSResource) resources.remove(uri);
    }

    public Map getAllResources() {
        return resources;
    }

    public void removeAllResources() {
        resources.clear();
    }

    //1. rdfs classes
    public static final int CLASS = 1;

    private Map classes = new HashMap();

    public void addClass(RDFSClass cls) {
        String uri = cls.getNamespace().getURI() + cls.getLocalName();
        addClass(uri, cls);
    }

    public void addClass(String uri, RDFSClass cls) {
        classes.put(uri, cls);
    }

    public RDFSClass getClass(String uri) {
        return (RDFSClass) classes.get(uri);
    }

    public RDFSClass removeClass(String uri) {
        return (RDFSClass) classes.remove(uri);
    }

    public void removeAllClasses() {
        classes.clear();
    }

    //2. rdf properties
    public static final int PROPERTY = 2;

    private Map properties = new HashMap();

    public void addProperty(RDFProperty property) {
        String uri = property.getNamespace().getURI() + property.getLocalName();
        addProperty(uri, property);
    }

    public void addProperty(String uri, RDFProperty property) {
        properties.put(uri, property);
    }

    public RDFProperty getProperty(String uri) {
        return (RDFProperty) properties.get(uri);
    }

    public RDFProperty removeProperty(String uri) {
        return (RDFProperty) properties.remove(uri);
    }

    public void removeAllProperties() {
        properties.clear();
    }

    //3. rdf refied refiedstatements
    public static final int REFIED_STATEMENT = 3;

    private Map refiedStatements = new HashMap();

    public void addRefiedStatement(RDFStatement statement) {
        String uri = statement.getNamespace().getURI()
                     + statement.getLocalName();
        addRefiedStatement(uri, statement);
    }

    public void addRefiedStatement(String uri, RDFStatement statement) {
        refiedStatements.put(uri, statement);
    }

    public RDFStatement getRefiedStatement(String uri) {
        return (RDFStatement) refiedStatements.get(uri);
    }

    public RDFStatement removeRefiedStatement(String uri) {
        return (RDFStatement) refiedStatements.remove(uri);
    }

    public void removeAllRefiedStatements() {
        refiedStatements.clear();
    }

    public Map getAllRefiedStatements() {
        return refiedStatements;
    }

    //4. rdfs containers
    public static final int CONTAINER = 4;

    private Map containers = new HashMap();

    public void addContainer(RDFSContainer container) {
        String uri = container.getNamespace().getURI()
                     + container.getLocalName();
        addContainer(uri, container);
    }

    public void addContainer(String uri, RDFSContainer container) {
        containers.put(uri, container);
    }

    public RDFSContainer getContainer(String uri) {
        return (RDFSContainer) containers.get(uri);
    }

    public RDFSContainer removeContainer(String uri) {
        return (RDFSContainer) containers.remove(uri);
    }

    public void removeAllContainers() {
        containers.clear();
    }

    public Map getAllContainers() {
        return containers;
    }

    //5. rdf lists
    public static final int LIST = 5;

    private Map lists = new HashMap();

    public void addList(RDFList list) {
        String uri = list.getNamespace().getURI() + list.getLocalName();
        addList(uri, list);
    }

    public void addList(String uri, RDFList list) {
        lists.put(uri, list);
    }

    public RDFList getList(String uri) {
        return (RDFList) lists.get(uri);
    }

    public RDFList removeList(String uri) {
        return (RDFList) lists.remove(uri);
    }

    public void removeAllLists() {
        lists.clear();
    }

    public Map getAllLists() {
        return lists;
    }

    //6. rdfs datatypes
    public static final int DATATYPE = 6;

    private Map datatypes = new HashMap();

    public void addDatatype(RDFSDatatype datatype) {
        String uri = datatype.getNamespace().getURI() + datatype.getLocalName();
        addDatatype(uri, datatype);
    }

    public void addDatatype(String uri, RDFSDatatype datatype) {
        datatypes.put(uri, datatype);
    }

    public RDFSDatatype getDatatype(String uri) {
        return (RDFSDatatype) datatypes.get(uri);
    }

    public RDFSDatatype removeDatatype(String uri) {
        return (RDFSDatatype) datatypes.remove(uri);
    }

    public void removeAllDatatypes() {
        datatypes.clear();
    }

    /* All resources - all Hashmaps */
    private Map[] allResources = new Map[] { resources, classes, properties,
            refiedStatements, containers, lists, datatypes };

    /**
     * Find the resource in all maps in the order given by the parameter order[]
     * 
     * @param uri
     *            the resource uri to find
     * @param order
     *            the order in which to find the global resource
     * @return the resource
     */
    public RDFSResource getGlobalResource(String uri, int[] order) {
        if (order == null || order.length != allResources.length)
            return null;

        RDFSResource resource = null;

        for (int i = 0; i < order.length; i++) {
            Object found = allResources[order[i]].get(uri);
            if (found != null) {
                resource = (RDFSResource) found;
                break;
            }
        }

        return resource;
    }

    /**
     * Remove all Hashmap elements
     */
    public void removeAll() {
        for (int i = 0; i < allResources.length; i++)
            allResources[i].clear();
        statements.clear();
    }

    //fact statements (not refication)
    private Map statements = new HashMap();

    public void addStatement(RDFStatement statement) {
        String uri = statement.getNamespace().getURI()
                     + statement.getLocalName();
        addStatement(uri, statement);
    }

    public void addStatement(String uri, RDFStatement statement) {
        statements.put(uri, statement);
    }

    public RDFStatement getStatement(String uri) {
        return (RDFStatement) statements.get(uri);
    }

    public RDFStatement removeStatement(String uri) {
        return (RDFStatement) statements.remove(uri);
    }

    public void removeAllStatements() {
        statements.clear();
    }

    public Map getAllStatements() {
        return statements;
    }
}