/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFTripleAnalyser.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdf.resource.parser.wrapper;

import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.eclipse.eodm.rdf.resource.parser.element.RDFLiteralElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFTriple;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.Namespace;
import org.eclipse.eodm.rdfs.Ontology;
import org.eclipse.eodm.rdfs.PlainLiteral;
import org.eclipse.eodm.rdfs.RDFList;
import org.eclipse.eodm.rdfs.RDFSContainer;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFStatement;
import org.eclipse.eodm.rdfs.TypedLiteral;
import org.eclipse.eodm.rdfs.vocabulary.RDF;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;

/**
 * RDFTripleAnalyser
 */
public class RDFTripleAnalyser {
    /* The namespace map */
    private RDFNamespaceMap namespaceMap = new RDFNamespaceMap();

    /**
     * Get the namespaces map
     * 
     * @return the namespaces map
     */
    public RDFNamespaceMap getNamespaceMap() {
        return namespaceMap;
    }

    /* The RDF Model: RDFDescription */
    private Ontology description = RDFSFactory.eINSTANCE.createOntology();

    /**
     * Get the EODM model's RDFDescription.
     * 
     * @return EODM model
     */
    public Ontology getDescription() {
        return description;
    }

    /* The resource mapping repository */
    private RDFResourceMap repository = new RDFResourceMap();

    /**
     * Get the Map Repository.
     * 
     * @return Map Repository
     */
    public RDFResourceMap getRepository() {
        return repository;
    }

    /**
     * Initialize the analyser. Add default namespaces to mapping repository and
     * RDF Model: RDFDescription
     */
    public void initialize() {
        handleNamespace("rdf", RDF.NAMESPACE);
        handleNamespace("rdfs", RDFS.NAMESPACE);
        handleNamespace("xsd", RDFS.XSD_NAMESPACE);
    }

    /**
     * Handle namespaces generated during RDF parsing.
     * 
     * @param name
     *            The abbreviation of the namespace
     * @param uri
     *            The namespace full uri
     * @return the Namespace handled
     */
    public Namespace handleNamespace(String name, String uri) {
        Namespace namespace = namespaceMap.getNamespace(uri);
        /* if the namespace is not found */
        /*
         * or it's not the BNode namespace && names are not equivalent, then add
         * it
         */
        if (namespace == null
            || !name.equals(namespace.getName())
            && !uri.equals(RDFS.BNODE_NAMESPACE)) {
            namespace = createNamespace(name, uri);
            //if the namespace's name is null, repository will give it a
            // default name
            namespaceMap.addNamespace(namespace);
            description.getOwnedNamespace().add(namespace);
        }
        return namespace;
    }

    /**
     * Create a new namespace with the name and uri designated.
     * 
     * @param name
     *            The namespace name
     * @param uri
     *            The namespace uri
     * @return The namespace created
     */
    private Namespace createNamespace(String name, String uri) {
        Namespace namespace = RDFSFactory.eINSTANCE.createNamespace();
        namespace.setName(name);
        namespace.setURI(uri);
        return namespace;
    }

    /**
     * Get a namespace by the designated uri. Note: the namespace's abbreviation
     * name - uri mapping relationship defined in source rdf file is lost.
     * Namespaces defined in one XML Node will be added into the RDFNamespaceMap
     * with a temporary abbreviation name.
     * 
     * @param uri
     *            The namespace uri
     * @return The namespace object
     */
    public Namespace getNamespaceByURI(String uri) {
        Namespace namespace = namespaceMap.getNamespace(uri);
        if (namespace == null)
            namespace = handleNamespace(null, uri);
        return namespace;
    }

    /**
     * Analyse an RDF triple {subject, predicate, object}, it's the first step.
     * 
     * @param subject
     *            The subject of the triple
     * @param predicate
     *            The predicate of the triple
     * @param object
     *            The object of the triple
     */
    public void analyseTriple(RDFResourceElement subject,
            URIReference predicate, RDFValue object) {
        //1. Just identify all typed triples (sub rdf:type obj)
        boolean handled = RDFTypeIdentifier._INSTANCE.identifyRDFType(subject,
                predicate, object, this);
        if (handled)
            return;

        //2. Handle Collection triples (subject rdf:first/rdf:rest object)
        handled = RDFCollectionConstructor._INSTANCE.handleRDFListTriple(
                subject, predicate, object, this);
        if (handled)
            return;

        //3. Handle triples with typed subject & object(type can be implicitly
        // identified)
        handled = RDFClassPropertyAnalyser._INSTANCE.handleRDFProperty(subject,
                predicate, object, this);
        if (handled)
            return;

        //4. Record unanalyzed triples
        addUnAnalyzedTriple(subject, predicate, object);
    }

    /* To store all unanalyzed rdf triples */
    private Vector unAnalyzedTriples = new Vector();

    private void addUnAnalyzedTriple(RDFResourceElement subject,
            URIReference predicate, RDFValue object) {
        unAnalyzedTriples.add(new RDFTriple(subject, predicate, object));
    }

    /**
     * Analyse all the remained (unanalyzed) triples, it's the second step.
     */
    public void analyzeRemainedTriples() {
        //1. utility triples and other common facts
        //rdf:value rdfs:label rdfs:comment rdfs:member
        //rdfs:seeAlso rdfs:isDefinedBy
        handleAllUnanalyzedTriples();

        //2. list
        //rdf:first
        handleListFirstResource();

        //3. container
        //rdf:_n
        handleRDFSMemberResource();

        //4. statement
        //rdf:subject rdf:predicate rdf:object
        handleRefiedStatementResource();
    }

    /**
     * 1. rdf:value rdfs:label rdfs:comment rdfs:member rdfs:seeAlso
     * rdfs:isDefinedBy 2. some common facts
     */
    private void handleAllUnanalyzedTriples() {
        for (int i = 0; i < unAnalyzedTriples.size(); i++) {
            RDFTriple triple = (RDFTriple) unAnalyzedTriples.get(i);
            RDFResourceElement subject = triple.getSubject();
            URIReference predicate = triple.getPredicate();
            RDFValue object = triple.getObject();

            //1. Handle container membership property(subject rdf:_n object)
            boolean analysed = RDFContainerConstructor._INSTANCE
                    .handleRDFSContainerTriple(subject, predicate, object, this);
            if (analysed)
                continue;

            //2. Handle rdfs:range property
            analysed = RDFClassPropertyAnalyser._INSTANCE
                    .handleRDFPropertyRange(subject, predicate, object, this);
            if (analysed)
                continue;

            //3. Handle rdf utility properties
            analysed = RDFUtilityPropertyAnalyser._INSTANCE
                    .handleRDFUtilityTriple(subject, predicate, object, this);
            if (analysed)
                continue;

            //4. Handle common facts
            handleFactTriple(subject, predicate, object);
        }
    }

    /**
     * Handle fact resources
     * 
     * @param subject
     *            the subject resource
     * @param predicate
     *            the predicate property
     * @param object
     *            the object resource
     */
    private void handleFactTriple(RDFResourceElement subject,
            URIReference predicate, RDFValue object) {
        RDFSResource subRes = RDFTypeIdentifier._INSTANCE
                .constructRDFSResource(subject.getNamespace(), subject
                        .getLocalName(), this);
        RDFSResource preRes = RDFTypeIdentifier._INSTANCE.constructRDFProperty(
                predicate.getNamespace(), predicate.getLocalName(), this);

        RDFSResource objRes = null;
        if (object instanceof RDFLiteralElement)
            objRes = createLiteral((RDFLiteralElement) object);
        else
            objRes = RDFTypeIdentifier._INSTANCE.constructRDFSResource(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), this);

        String statementLocalName = ""
                                    + subRes.hashCode() + predicate.hashCode()
                                    + objRes.hashCode();
        RDFStatement statement = getRepository().getStatement(
                statementLocalName);
        if (statement == null) {
            statement = RDFTypeGenerator._INSTANCE
                    .createStatement(getNamespaceByURI(RDFS.BNODE_NAMESPACE),
                            statementLocalName);
            statement.setReified(false);
            statement.setRDFSubject(subRes);
            statement.setRDFPredicate(preRes);
            statement.setRDFObject(objRes);
            getRepository().addStatement(statementLocalName, statement);
            getDescription().getContains().add(statement);
        }
    }

    /**
     * Create a literal and add it to RDFModel: RDFDescription
     * 
     * @param rdfLiteral
     *            the rdf literal
     * @return the RDFSLiteral
     */
    public RDFSLiteral createLiteral(RDFLiteralElement rdfLiteral) {
        RDFSLiteral literal = null;
        URIReference typeRef = rdfLiteral.getDatatype();
        if (typeRef != null) {
            if (typeRef.getFullURI().equals(RDF.NAMESPACE + RDF.C_XMLLITERAL))
                literal = RDFSFactory.eINSTANCE.createRDFXMLLiteral();
            else
                literal = RDFSFactory.eINSTANCE.createTypedLiteral();
            RDFSDatatype type = RDFTypeIdentifier._INSTANCE
                    .constructRDFSDatatype(typeRef.getNamespace(), typeRef
                            .getLocalName(), this);
            ((TypedLiteral) literal).setDatatype(type);
        } else if (rdfLiteral.getLanguage() != null) {
            literal = RDFSFactory.eINSTANCE.createPlainLiteral();
            ((PlainLiteral) literal).setLanguage(rdfLiteral.getLanguage());
        } else
            literal = RDFSFactory.eINSTANCE.createRDFSLiteral();
        literal.setLexicalForm(rdfLiteral.getText());
        getDescription().getContains().add(literal);
        return literal;
    }

    /**
     * Handle RDFList's first resource rdf:first
     */
    private void handleListFirstResource() {
        Map lists = getRepository().getAllLists();
        Iterator keys = lists.keySet().iterator();
        while (keys.hasNext()) {
            RDFList list = (RDFList) lists.get(keys.next());
            RDFCollectionConstructor._INSTANCE.handleRDFListFirstElelement(
                    list, this);
        }
    }

    /**
     * Handle RDFSResource's member resource 1. RDFSContainers 2. RDFSResources
     * rdf:_n
     */
    private void handleRDFSMemberResource() {
        Map containers = getRepository().getAllContainers();
        Iterator containerKeys = containers.keySet().iterator();
        while (containerKeys.hasNext()) {
            RDFSContainer container = (RDFSContainer) containers
                    .get(containerKeys.next());
            RDFContainerConstructor._INSTANCE
                    .handleRDFSMembers(container, this);
        }

        Map resources = getRepository().getAllResources();
        Iterator resourceKeys = resources.keySet().iterator();
        while (resourceKeys.hasNext()) {
            RDFSResource resource = (RDFSResource) resources.get(resourceKeys
                    .next());
            RDFContainerConstructor._INSTANCE.handleRDFSMembers(resource, this);
        }
    }

    /**
     * Handle refied statement's subject, predicate, object rdf:subject,
     * rdf:predicate, rdf:object
     */
    private void handleRefiedStatementResource() {
        Map statements = getRepository().getAllRefiedStatements();
        Iterator keys = statements.keySet().iterator();
        while (keys.hasNext()) {
            RDFStatement statement = (RDFStatement) statements.get(keys.next());
            //subject
            if (statement.getRDFSubject() != null
                && !(statement.getRDFSubject() instanceof RDFSLiteral))
                statement.setRDFSubject(RDFTypeIdentifier._INSTANCE
                        .constructRDFSResource(statement.getRDFSubject()
                                .getNamespace().getURI(), statement
                                .getRDFSubject().getLocalName(), this));
            //predicate
            if (statement.getRDFPredicate() != null
                && !(statement.getRDFPredicate() instanceof RDFSLiteral))
                statement.setRDFPredicate(RDFTypeIdentifier._INSTANCE
                        .constructRDFProperty(statement.getRDFPredicate()
                                .getNamespace().getURI(), statement
                                .getRDFPredicate().getLocalName(), this));
            //object
            if (statement.getRDFObject() != null
                && !(statement.getRDFObject() instanceof RDFSLiteral))
                statement.setRDFObject(RDFTypeIdentifier._INSTANCE
                        .constructRDFSResource(statement.getRDFObject()
                                .getNamespace().getURI(), statement
                                .getRDFObject().getLocalName(), this));
        }
    }

    /**
     * The analyzing process ended.
     */
    public void endAnalyzing() {
    }

    /**
     * Clear all the temporary resourced created during parsing process.
     */
    public void clearResources() {
        //remove all unanalysed triple
        unAnalyzedTriples.removeAllElements();
        //remove all namespacces
        namespaceMap.removeAllNamespaces();
        //remove all maps elements
        getRepository().removeAll();
    }
}