/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFS2ECoreTransformer.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdfs.transformer.ecore;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.eodm.rdf.resource.RDFXMLResourceFactoryImpl;
import org.eclipse.eodm.rdfs.Ontology;
import org.eclipse.eodm.rdfs.RDFBag;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.transformer.RDFSTransformationException;
import org.eclipse.eodm.rdfs.vocabulary.RDF;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;
import org.eclipse.eodm.rdfs.vocabulary.XSD;

/**
 * The API realizes translation from RDFS model to ECore model.
 *  
 */
public class RDFS2ECoreTransformer {

    //input file
    private String inputRDFFile = null;

    //output file
    private String outputEcoreFile = null;

    //RDF
    private RDFSFactory factory = null;

    //ontology model
    private Ontology ontology;

    //Ecore
    private EcoreFactory ecoreFactory = null;

    private EcorePackage ecorePackage = null;

    //Ecore pacakge
    private EPackage epackage = null;

    private String name = null;

    private String uri = null;

    private String nsPrefix = null;

    public RDFS2ECoreTransformer() {

        factory = RDFSFactory.eINSTANCE;
        // create model
        ontology = factory.createOntology();

        ecoreFactory = EcoreFactory.eINSTANCE;
        ecorePackage = EcorePackage.eINSTANCE;
        epackage = ecoreFactory.createEPackage();

    }

    /**
     * Translate function Translate the input rdf file to target ecore file.
     * 
     * @param input
     * @throws RDFSTransformationException
     */
    public void convertRDFS2Ecore(String input) throws RDFSTransformationException {
        convertRDFS2Ecore(input, null);
    }

    /**
     * Translate function Translate the input rdf file to target ecore file.
     * 
     * @param input
     *            rdf file
     * @param output
     *            target ecore file
     * @throws RDFSTransformationException
     */
    public void convertRDFS2Ecore(String input, String output)
            throws RDFSTransformationException {

        //check input params
        if (input == null) {
            RDFSTransformationException e = new RDFSTransformationException(
                    "Initate mapping failed : input RDF model is NULL.");
            throw e;
        }

        // check and set input and output files
        if (input != null)
            if (input.equals("") == false)
                setInputRDFFile(input);
        if (output != null)
            if (output.equals("") == false)
                setOutputEcoreFile(output);

        //create RDFS model and Ecore model
        createRDFSModel();
        createEcoreModel();

        //RDFS -> Ecore

        //RDFSClass -> EClass
        List classList = ontology.getResourceList(RDFSClass.class);
        Iterator ri = classList.iterator();

        while (ri.hasNext()) {
            RDFSClass c = (RDFSClass) ri.next();

            if (c instanceof RDFSDatatype)
                continue;
            EClass eclass = ecoreFactory.createEClass();
            eclass.setName(c.getLocalName());
            createAnnotation(ecoreFactory, eclass, c);
            epackage.getEClassifiers().add(eclass);
        }

        //RDFSClass.RDFSSubClassOf -> EClass.eSuperType
        ri = classList.iterator();
        while (ri.hasNext()) {
            RDFSClass c = (RDFSClass) ri.next();
            EClass cclass = (EClass) epackage.getEClassifier(c.getLocalName());

            if (cclass == null)
                continue;

            Iterator ssc = c.getRDFSSubClassOf().iterator();
            while (ssc.hasNext()) {
                RDFSClass superClass = (RDFSClass) ssc.next();
                cclass.getESuperTypes().add(
                        (EClass) epackage.getEClassifier(superClass
                                .getLocalName()));
            }
        }

        //RDFSDatatype -> EDataType, but XSD datatypes will not be transformed
        ri = ontology.getResourceList(RDFSDatatype.class).iterator();
        HashMap dataTypeBags = new HashMap();
        while (ri.hasNext()) {
            RDFSDatatype dt = (RDFSDatatype) (ri.next());
            if (dt.getURI() == null)
                continue;
            // skip XSD datatypes and Ecore datatypes
            if (dt.getURI().equals(XSD.NAMESPACE + XSD.sxboolean)
                || dt.getURI().equals(XSD.NAMESPACE + XSD.sxfloat)
                || dt.getURI().equals(XSD.NAMESPACE + XSD.sxbyte)
                || dt.getURI().equals(XSD.NAMESPACE + XSD.sxint)
                || dt.getURI().equals(XSD.NAMESPACE + XSD.sxlong)
                || dt.getURI().equals(XSD.NAMESPACE + XSD.sxdouble)
                || dt.getURI().equals(XSD.NAMESPACE + XSD.sxshort)
                || dt.getURI().equals(XSD.NAMESPACE + XSD.sxstring)
                || dt.getURI().equals(XSD.NAMESPACE + XSD.sinteger)
                || dt.getURI().indexOf(XSD.ECORE_NAMESPACE) > -1)
                continue;

            //			 if (dt.getLocalName().startsWith("Bag_")){
            //				 boolean stop = true;
            //			 }
            if (RDFBag.class
                    .isInstance((dt.getRDFSIsDefinedBy().size() > 0 ? dt
                            .getRDFSIsDefinedBy().get(0) : null))) {

                // RDFBag -> EEnum
                dataTypeBags.put((RDFBag) dt.getRDFSIsDefinedBy().get(0),
                        createEEnum((RDFBag) dt.getRDFSIsDefinedBy().get(0)));

            } else {
                // Create a user-defined EDatatType
                EDataType ed = ecoreFactory.createEDataType();
                ed.setName(dt.getLocalName());
                createAnnotation(ecoreFactory, ed, dt);
                epackage.getEClassifiers().add(ed);
            }
        }

        // RDFBag -> EEnum
        List bags = ontology.getResourceList(RDFBag.class);
        // remove datatype bags
        bags.removeAll(dataTypeBags.keySet());

        ri = bags.iterator();
        while (ri.hasNext()) {
            createEEnum((RDFBag) ri.next());
        }

        // RDFProperty -> EAttribute & EReference
        ri = classList.iterator();
        while (ri.hasNext()) {
            RDFSClass r = (RDFSClass) ri.next();
            EClass eclass = null;
            if (!EDataType.class.isInstance(epackage.getEClassifier(r
                    .getLocalName())))
                eclass = (EClass) epackage.getEClassifier(r.getLocalName());
            if (eclass == null)
                continue;

            Iterator pi = r.getOwnedProperty().iterator();
            while (pi.hasNext()) {
                RDFProperty p = (RDFProperty) (pi.next());
                RDFSResource ran = null;
                if (p.getRDFSRange().iterator().hasNext() == false) {
                    // if property without range, use String for default
                    EAttribute attribute = ecoreFactory.createEAttribute();
                    createAnnotation(ecoreFactory, attribute, p);
                    attribute.setName(p.getLocalName());
                    attribute.setEType(ecorePackage.getEString());
                    eclass.getEStructuralFeatures().add(attribute);
                } else {
                    // use only the first range
                    ran = (RDFSResource) p.getRDFSRange().iterator().next();

                    // RDFSDatatype
                    if (ran instanceof RDFSDatatype) {

                        EAttribute attribute = ecoreFactory.createEAttribute();
                        createAnnotation(ecoreFactory, attribute, p);
                        attribute.setName(p.getLocalName());
                        if (ran.getURI() != null) {
                            //9 kind mapping: XSD<->EMF
                            if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sxboolean))
                                attribute.setEType(ecorePackage.getEBoolean());
                            else if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sxfloat))
                                attribute.setEType(ecorePackage.getEFloat());
                            else if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sxbyte))
                                attribute.setEType(ecorePackage.getEByte());
                            else if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sxint))
                                attribute.setEType(ecorePackage.getEInt());
                            else if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sxlong))
                                attribute.setEType(ecorePackage.getELong());
                            else if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sxdouble))
                                attribute.setEType(ecorePackage.getEDouble());
                            else if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sxshort))
                                attribute.setEType(ecorePackage.getEShort());
                            else if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sinteger))
                                attribute.setEType(ecorePackage
                                        .getEIntegerObject());
                            else if (ran.getURI().equals(
                                    XSD.NAMESPACE + XSD.sxstring)
                                     || ran.getLocalName().endsWith(
                                             RDFS.C_LITERAL)
                                     | ran.getLocalName().endsWith(
                                             RDF.C_XMLLITERAL))
                                attribute.setEType(ecorePackage.getEString());
                            else {
                                // user-defined datatype
                                if (RDFBag.class.isInstance((ran
                                        .getRDFSIsDefinedBy().size() > 0 ? ran
                                        .getRDFSIsDefinedBy().get(0) : null))) {
                                    // Set EEnum datatype
                                    attribute.setEType((EEnum) dataTypeBags
                                            .get(ran.getRDFSIsDefinedBy()
                                                    .get(0)));
                                } else {
                                    attribute
                                            .setEType(epackage
                                                    .getEClassifier(ran
                                                            .getLocalName()));
                                }
                                EAnnotation an = ecoreFactory
                                        .createEAnnotation();
                                an.setSource(RDFS.C_DATATYPE);
                                attribute.getEAnnotations().add(an);
                            }
                        }
                        eclass.getEStructuralFeatures().add(attribute);
                    } else {
                        if (ran instanceof RDFSClass) {
                            EReference reference = ecoreFactory
                                    .createEReference();
                            reference.setName(p.getLocalName());
                            createAnnotation(ecoreFactory, reference, p);
                            reference.setEType(epackage.getEClassifier(ran
                                    .getLocalName()));
                            eclass.getEStructuralFeatures().add(reference);
                        }
                    }
                }
            }
        }

        //save ecore model
        ResourceSet resourceSet = new ResourceSetImpl();
        resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
                .put(Resource.Factory.Registry.DEFAULT_EXTENSION,
                        new EcoreResourceFactoryImpl());

        Resource resource = resourceSet.createResource(URI
                .createFileURI(new File(outputEcoreFile).getAbsolutePath()));
        resource.getContents().add(epackage);
        try {
            resource.save(Collections.EMPTY_MAP);
        } catch (Exception e) {
            RDFSTransformationException te = new RDFSTransformationException(
                    "Error during serialize into XMI file: " + e.toString());
            throw te;
        }

    }//main

    /**
     * set input rdf file's URL
     * 
     * @param in
     *            rdf file's URL
     */
    public void setInputRDFFile(String in) {

        if (in == null) {
            inputRDFFile = in;
            return;
        }
        inputRDFFile = in;

        if (outputEcoreFile == null) {
            int p = inputRDFFile.lastIndexOf(".");
            outputEcoreFile = (p != -1 ? inputRDFFile.substring(0, p + 1)
                                         + "ecore" : inputRDFFile + ".ecore");
        }

    }

    /**
     * Set properties for target ecore package
     * 
     * @param t_name
     *            package name
     * @param t_uri
     *            package namespace URI
     * @param t_nsPrefix
     *            pckage namespace prefix
     */
    public void setEPackageURI(String t_name, String t_uri, String t_nsPrefix) {

        name = t_name;
        uri = t_uri;
        nsPrefix = t_nsPrefix;
    }

    /**
     * Create EEnum from RDFBag
     * 
     * @param bag
     * @return
     */
    private EEnum createEEnum(RDFBag bag) {

        if (bag == null)
            return null;
        Iterator memberList = bag.getRDFSMember().iterator();
        EEnum eenum = null;
        // if bag.size > 0, create EEnum
        if (memberList.hasNext()) {
            eenum = ecoreFactory.createEEnum();
            eenum.setName(bag.getLocalName());
            createAnnotation(ecoreFactory, eenum, bag);
        }

        // create enumliterals from Bag members
        int intValue = 0;
        while (memberList.hasNext()) {
            RDFSResource r = (RDFSResource) (memberList.next());
            //test whether contains label "EEnum"
            String enumeration = (r instanceof RDFSLiteral ? ((RDFSLiteral) r)
                    .getLexicalForm() : r.getLocalName());

            // create EEnumLiteral
            EEnumLiteral eliteral = ecoreFactory.createEEnumLiteral();
            createAnnotation(ecoreFactory, eliteral, r);
            eliteral.setName(enumeration);
            eliteral.setValue(intValue);
            eenum.getELiterals().add(eliteral);
            intValue++;
        }
        epackage.getEClassifiers().add(eenum);
        return eenum;
    }

    /**
     * initiate the ecore model
     * 
     * @return
     */
    private boolean createEcoreModel() {

        if (outputEcoreFile == null)
            return false;

        // set up EPackage
        epackage.setName(name);
        epackage.setNsPrefix(nsPrefix);
        epackage.setNsURI(uri);
        EPackage.Registry.INSTANCE.put(uri, epackage);

        return true;

    }

    /**
     * load RDF file and create a RDF ontology model
     * 
     * @return
     * @throws RDFSTransformationException
     */
    private boolean createRDFSModel() throws RDFSTransformationException {

        if (inputRDFFile == null) {
            RDFSTransformationException e = new RDFSTransformationException(
                    "Create RDFSModel failed : inputfile not found.");
            throw e;
        }

        ResourceSet resourceSet = null;
        URI inFileURI = null;
        Resource inputResource = null;
        try {
            System.setProperty("org.xml.sax.driver",
                    "org.apache.xerces.parsers.SAXParser");

            resourceSet = new ResourceSetImpl();
            resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
                    .put(Resource.Factory.Registry.DEFAULT_EXTENSION,
                            new RDFXMLResourceFactoryImpl());
            inFileURI = URI.createFileURI(new File(inputRDFFile)
                    .getAbsolutePath());
            inputResource = resourceSet.createResource(inFileURI);
            inputResource.load(Collections.EMPTY_MAP);
            ontology = (inputResource.getContents().get(0) == null ? null
                    : (Ontology) inputResource.getContents().get(0));
            if (ontology == null) {
                RDFSTransformationException e = new RDFSTransformationException(
                        "Create RDFSModel failed : no RDFDescription in the input file.");
                throw e;
            }
        } catch (IOException e) {
            RDFSTransformationException te = new RDFSTransformationException(
                    "Create RDFSModel failed: " + e.getMessage());
            throw te;
            // return false;
        } finally {
            resourceSet = null;
            inFileURI = null;
            inputResource = null;
        }

        return true;
    }

    /**
     * create eAnnotations from comment, isDefinedBy and Label in RDFSResource
     * 
     * @param fact
     * @param ec
     * @param rs
     */
    private void createAnnotation(EcoreFactory fact, ENamedElement ec,
            RDFSResource rs) {

        if (fact == null || ec == null || rs == null)
            return;

        // check input params
        if (rs == null || ec == null || fact == null)
            return;

        // create comments as annotations
        Iterator it = rs.getRDFSComment().iterator();
        while (it.hasNext()) {
            if (ec instanceof EDataType) {
                String comment = ((RDFSLiteral) it.next()).getLexicalForm();
                if (comment.startsWith(XSD.ECORE_INSTANCE_CLASS_NAME)) {
                    ((EDataType) ec).setInstanceClassName(comment
                            .substring(XSD.ECORE_INSTANCE_CLASS_NAME.length()));
                    continue;
                }
            }
            EAnnotation anno = fact.createEAnnotation();
            anno.setSource("comment - "
                           + ((RDFSLiteral) it.next()).getLexicalForm());
            ec.getEAnnotations().add(anno);
        }

        // create isDefinedBy as annotations
        it = rs.getRDFSIsDefinedBy().iterator();
        while (it.hasNext()) {
            EAnnotation anno = fact.createEAnnotation();
            anno.setSource("isDefinedBy - "
                           + ((RDFSResource) it.next()).getURI());
            ec.getEAnnotations().add(anno);
        }

        // create Label as annotations
        it = rs.getRDFSLabel().iterator();
        while (it.hasNext()) {
            EAnnotation anno = fact.createEAnnotation();
            anno.setSource("label - "
                           + ((RDFSLiteral) it.next()).getLexicalForm());
            ec.getEAnnotations().add(anno);
        }

    }

    /**
     * @return
     */
    public String getOutputEcoreFile() {
        return outputEcoreFile;
    }

    /**
     * @param outputEcoreFile
     */
    public void setOutputEcoreFile(String outputEcoreFile) {
        if (outputEcoreFile == null)
            return;
        this.outputEcoreFile = outputEcoreFile;
    }

    /**
     * @return
     */
    public String getInputRDFFile() {
        return inputRDFFile;
    }

}
